%Instruments Correlations: Table 2
%Acemoglu, Garcia-Jimeno, Robinson

clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Import data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Open data file State_Capacity_Network_Work.mat
load State_Capacity_Network_Work.mat

 %Choose the endogenous and exogenous variables for the model
     Yg_input = (Yg(:,4));                                                    %Yg = Development Outcomes
     Yv_input = 0*Yv(:,1);
     S_input = S(:,2);
        
     C_input = [C(:,1) C(:,3)];                                               %C = Colonial State
     R_input = R;                                                             %R = Distance to royal roads
     X_input = [G(:,1:5) Xs(:,1:2) S(:,2) DD(:,1:25) Dc];                     %X = Controls
     %Ev_input = Ev;                                                          %Historical Electoral variability
     A_input = Adj;                                                           %A = Adjacency matrix
     Dd_input = Ddist;                                                        %Original Distances matrix
     Drr_input = Drankrange;                                                  %Original Range of elevation ranks along path matrix
     Drv_input = Drankvar;                                                    %Original Variance of elevation ranks along path matrix
     
     %Clean data from missing values
     [Yg_clean S_clean C_clean X_clean R_clean Yv_clean A_clean Dd_clean Drr_clean Drv_clean DaneCodes_clean] = clean_missings(Yg_input, S_input, C_input, X_input, R_input, Yv_input, A_input, Dd_input, Drr_input, Drv_input, DaneCodes);

 clear Adj C DD Dc Ddist Drankrange Drankvar G H L R S Xs Yg Ygini Ev DaneCodes
 clear Yg_input S_input C_input X_input R_input Ev_input A_input Dd_input Drr_input Drv_input

                      
 c = C_clean;                           %c=vector of colonial state presence
 r = log(R_clean);                      %r=vector of distance to royal roads
 
 global D E
 D= Dd_clean;                           %D=distance matrix
 E= Drv_clean;                          %E=variance elevation matrix

 global A
 A=A_clean;                             %A=adjacency matrix (nxn) with 0s on the diagonal

 n = size(A,1);
 
 DaneCodes=DaneCodes_clean;             %Municipality codes to keep track
 
 Depts = X_clean(:,9:33);
 
 clear Yg_clean S_clean C_clean X_clean R_clean A_clean Dd_clean Drr_clean Drv_clean DaneCodes_clean
 
 
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Assumptions on the Network Structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Choose assumption a or b
%Assumption a: Network link = Adjacency
%Assumption b: Network link = Adjecency and Adjecency^2

ass_network_structure=0; % 0=a ; 1=b

global NE N N_2 A_2 A_3
F=1./(1+(D.*(1+E)));
%F=1./(D.*(1+E));

%Re-sets the diagonal to zeroes
for i=1:size(F,1)
    F(i,i)=0;
end

Q=(ones(n,n)-A).*(A*A-diag(diag(A*A)));                 %Matrix of Number of Paths to Neighbors of Neighbors who are not own Neighbors
A_2=ismember(Q,1:1:max(max(Q)));                        %Adjacency Matrix of neighbors of neighbors
A_2=+A_2;                                               % from logical to numeric

Q2=(ones(n,n)-A-A_2).*((A*A*A)-diag(diag(A*A*A)));      %Matrix of Number of Paths to Neighbors of Neighbors of Neighbors who are not own Neighbors or own Neighbors of Neighbors
A_3 = ismember(Q2,1:1:max(max(Q2)));                    %Adjacency Matrix of neighbors of neighbors of neighbors
A_3=+A_3;                                               % from logical to numeric
clear Q Q2

%Network matrix of adjacent neighbors
N = F.*A;
N_2 = F.*A_2;



%%

N_r = ((N*r)./(sum(A)'));

N2_r = ((N_2*r)./(sum((A_2))'));

N_c1 = ((N*c(:,1))./(sum(A)'));

N2_c1 = ((N_2*c(:,1))./(sum((A_2))'));

N_c2 = ((N*c(:,2))./(sum(A)'));

N2_c2 = ((N_2*c(:,2))./(sum((A_2))'));

X = [r N_r N2_r c(:,1) N_c1 N2_c1 c(:,2) N_c2 N2_c2];

X_1 = X(Depts(:,1)==1,:);
X_2 = X(Depts(:,2)==1,:);
X_3 = X(Depts(:,3)==1,:);
X_4 = X(Depts(:,4)==1,:);
X_5 = X(Depts(:,5)==1,:);
X_6 = X(Depts(:,6)==1,:);
X_7 = X(Depts(:,7)==1,:);
X_8 = X(Depts(:,8)==1,:);
X_9 = X(Depts(:,9)==1,:);
X_10 = X(Depts(:,10)==1,:);
X_11 = X(Depts(:,11)==1,:);
X_12 = X(Depts(:,12)==1,:);
X_13 = X(Depts(:,13)==1,:);
X_14 = X(Depts(:,14)==1,:);
X_15 = X(Depts(:,15)==1,:);
X_16 = X(Depts(:,16)==1,:);
X_17 = X(Depts(:,17)==1,:);
X_18 = X(Depts(:,18)==1,:);
X_19 = X(Depts(:,19)==1,:);
X_20 = X(Depts(:,20)==1,:);
X_21 = X(Depts(:,21)==1,:);
X_22 = X(Depts(:,22)==1,:);
X_23 = X(Depts(:,23)==1,:);
X_24 = X(Depts(:,24)==1,:);
X_25 = X(Depts(:,25)==1,:);

corr_X1 = corr(X_1);
corr_X1(isnan(corr_X1))=0;
corr_X1(eye(size(corr_X1))~=0)=1;

corr_X2 = corr(X_2);
corr_X2(isnan(corr_X2))=0;
corr_X2(eye(size(corr_X2))~=0)=1;

corr_X3 = corr(X_3);
corr_X3(isnan(corr_X3))=0;
corr_X3(eye(size(corr_X3))~=0)=1;

corr_X4 = corr(X_4);
corr_X4(isnan(corr_X4))=0;
corr_X4(eye(size(corr_X4))~=0)=1;

corr_X5 = corr(X_5);
corr_X5(isnan(corr_X5))=0;
corr_X5(eye(size(corr_X5))~=0)=1;

corr_X6 = corr(X_6);
corr_X6(isnan(corr_X6))=0;
corr_X6(eye(size(corr_X6))~=0)=1;

corr_X7 = corr(X_7);
corr_X7(isnan(corr_X7))=0;
corr_X7(eye(size(corr_X7))~=0)=1;

corr_X8 = corr(X_8);
corr_X8(isnan(corr_X8))=0;
corr_X8(eye(size(corr_X8))~=0)=1;

corr_X9 = corr(X_9);
corr_X9(isnan(corr_X9))=0;
corr_X9(eye(size(corr_X9))~=0)=1;

corr_X10 = corr(X_10);
corr_X10(isnan(corr_X10))=0;
corr_X10(eye(size(corr_X10))~=0)=1;

corr_X11 = corr(X_11);
corr_X11(isnan(corr_X11))=0;
corr_X11(eye(size(corr_X11))~=0)=1;

corr_X12 = corr(X_12);
corr_X12(isnan(corr_X12))=0;
corr_X12(eye(size(corr_X12))~=0)=1;

corr_X13 = corr(X_13);
corr_X13(isnan(corr_X13))=0;
corr_X13(eye(size(corr_X13))~=0)=1;

corr_X14 = corr(X_14);
corr_X14(isnan(corr_X14))=0;
corr_X14(eye(size(corr_X14))~=0)=1;

corr_X15 = corr(X_15);
corr_X15(isnan(corr_X15))=0;
corr_X15(eye(size(corr_X15))~=0)=1;

corr_X16 = corr(X_16);
corr_X16(isnan(corr_X16))=0;
corr_X16(eye(size(corr_X16))~=0)=1;

corr_X17 = corr(X_17);
corr_X17(isnan(corr_X17))=0;
corr_X17(eye(size(corr_X17))~=0)=1;

corr_X18 = corr(X_18);
corr_X18(isnan(corr_X18))=0;
corr_X18(eye(size(corr_X18))~=0)=1;

corr_X19 = corr(X_19);
corr_X19(isnan(corr_X19))=0;
corr_X19(eye(size(corr_X19))~=0)=1;

corr_X20 = corr(X_20);
corr_X20(isnan(corr_X20))=0;
corr_X20(eye(size(corr_X20))~=0)=1;

corr_X21 = corr(X_21);
corr_X21(isnan(corr_X21))=0;
corr_X21(eye(size(corr_X21))~=0)=1;

corr_X22 = corr(X_22);
corr_X22(isnan(corr_X22))=0;
corr_X22(eye(size(corr_X22))~=0)=1;

corr_X23 = corr(X_23);
corr_X23(isnan(corr_X23))=0;
corr_X23(eye(size(corr_X23))~=0)=1;

corr_X24 = corr(X_24);
corr_X24(isnan(corr_X24))=0;
corr_X24(eye(size(corr_X24))~=0)=1;

corr_X25 = corr(X_25);
corr_X25(isnan(corr_X25))=0;
corr_X25(eye(size(corr_X25))~=0)=1;


w=sum(Depts)/sum(sum(Depts));


Average_corr=w(1)*corr_X1 + w(2)*corr_X2 + w(3)*corr_X3 + w(4)*corr_X4 + w(5)*corr_X5 + w(6)*corr_X6 + w(7)*corr_X7 + w(8)*corr_X8 + w(9)*corr_X9 + w(10)*corr_X10 + w(11)*corr_X11 + w(12)*corr_X12 + w(13)*corr_X13 + w(14)*corr_X14 + w(15)*corr_X15 + w(16)*corr_X16 + w(17)*corr_X17 + w(18)*corr_X18 + w(19)*corr_X19 + w(20)*corr_X20 + w(21)*corr_X21 + w(22)*corr_X22 + w(23)*corr_X23 + w(24)*corr_X24 + w(25)*corr_X25; 

disp('Within Department Spacial Correlations of Instruments')
disp(Average_corr);




