%Estimation of State Capacity Spillovers: SIMULATED GMM GENERAL MODEL
%Acemoglu, Garcia-Jimeno, Robinson

clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Import data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Open data file State_Capacity_Network_Work.mat
load State_Capacity_Network_Work.mat

%%
%OPTIONS

%1: Choose Outcome variable:
 %out = 3;          %1 = Secondary enrollment 92-02
                    %2 = Services Coverage 1998
                    %3 = Not in poverty 2005
                    %4 = Life Quality Index
                    %5 = Primary enrollment 92-02 (placebo)
                    %6 = Vaccination coverage 1998 (placebo)    
                                       
%2: Choose Measure of State Capacity:
 a_or_b = 0;        %0 = State municipality agencies 
                    %1 = State municipality bureaucracies
 
%3: Choose if control for national-level bureaucracy 
 nat_bur=0;         %0 = no 
                    %1 = yes

%4: Choose if control for rivers density and land shares
 river_land=0;      %0=no
                    %1=yes

%5: Choose if control for distance to current road
current_road=1;     %0=no
                    %1=yes
                    
%6: Choose assumption on Network Links
 ass_network_structure=0;       %0 = Network link is neighbors           
                                %1 = Network link is neighbors and neighbors of neighbors 

%7: Choose order of instruments: 
 order=2;           %2 = Instruments are Nc and N2c
                    %3 = Instruments are N2c and N3c 

%8: Choose the subset of Instruments to use:
 instruments=0;     %0 = Royal roads and Colonial Agencies
                    %1 = Royal roads only
                    %2 = Colonial Agencies only

%9: Choose if exclude violent municipalities
excl_viol=0;        %0 = Do not exclude
                    %1 = Exclude from sample
                    %2 = Exclude from network

%10: Choose how to treat population
pop_treat=0;        %0=Instrument for Population with Historical Population
                    %1=Control for Population                    
%%

 %Choose the endogenous and exogenous variables for the model
 Yg_input = [Yg(:,2) ((Yg(:,4)+Yg(:,5)+Yg(:,6))/3) Yg(:,9) Yg(:,10)];    
         
     %Choose if state capacity is agencies or bureaucracies    
    if a_or_b ==0
         S_input = [S(:,6)+S(:,7)+S(:,8)+S(:,11)+S(:,12)+S(:,13)+S(:,14)+S(:,15)+S(:,16)+S(:,18)+S(:,19) S(:,2)];                 %S = Contemporary State
     else
         S_input = [S(:,1) S(:,2)];
     end
         
     C_input = [C(:,1) C(:,3)];                                               %C = Colonial State
     R_input = R;                                                             %R = Distance to royal roads
     
     %Choose controls
     if nat_bur==0 && river_land==0 && current_road==0                        %X = Controls
         X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5)];
        else if nat_bur==0 && river_land==0 && current_road==1
                X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1)];               
            else if nat_bur==0 && river_land==1 && current_road==0
                    X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) G(:,6:8) L(:,1:7) L(:,12:15)];
                else if nat_bur==0 && river_land==1 && current_road==1
                        X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1) G(:,6:8) L(:,1:7) L(:,12:15)];
                    else if nat_bur==1 && river_land==0 && current_road==0
                            X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) log(1+S(:,2))];
                        else if nat_bur==1 && river_land==0 && current_road==1
                                X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1) log(1+S(:,2))];
                            else if nat_bur==1 && river_land==1 && current_road==0
                                    X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) G(:,6:8) L(:,1:7) L(:,12:15) log(1+S(:,2))];
                                else 
                                    X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1) G(:,6:8) L(:,1:7) L(:,12:15) log(1+S(:,2))];
                                end
                            end
                        end
                    end
                end
            end
     end        

     for i=1:size(NS,1)
         if NS(i,1)>0.1
             NS(i,1)=0.1;
         end
     end
             
     for i=1:size(NS,1)
         if NS(i,2)>500
             NS(i,2)=500;
         end
     end
     
                                                        
     X_input = [X_input Ev(:,1) zscore(NS) H(:,8)./1000 H(:,9)];      %To add 1843 population, missing 1843 pop dummy, Historical electoral variability and Network Statistics
     
     size_X_input = size(X_input,2);                                          %Number of covariates
     size_V = size(Ev(:,1),2) + size(NS,2);                                   %Number of covariates for unobserved national weights
     
     A_input = Adj;                                                           %A = Adjacency matrix
     Dd_input = Ddist;                                                        %Original Distances matrix
     Drr_input = Drankrange;                                                  %Original Range of elevation ranks along path matrix
     Drv_input = Drankvar;                                                    %Original Variance of elevation ranks along path matrix
     Yv_input = Yv(:,16);                                                     %Original Per capita attacks   
     
     
     %Clean data from missing values
     [Yg_clean S_clean C_clean X_clean R_clean Yv_clean A_clean Dd_clean Drr_clean Drv_clean DaneCodes_clean] = clean_missings(Yg_input, S_input, C_input, X_input, R_input, Yv_input, A_input, Dd_input, Drr_input, Drv_input, DaneCodes);

 clear Adj C DD Dc Ddist Drankrange Drankvar G H L R S Xs Yg Yv Ygini Ev DaneCodes NS
 clear Yg_input S_input C_input X_input R_input Yv_input Ev_input A_input Dd_input Drr_input Drv_input


 global p l b c r
 p = Yg_clean;                          %p=vectors of prosperity

 l= log(1+S_clean(:,1));                %l=vector of LOCAL state capacity  
 b= log(1+S_clean(:,2));                %b=vector of NATIONAL-LEVEL state capacity  
 
 c = (C_clean);                         %c=vector of colonial state presence
 r = zscore(R_clean);                   %r=vector of distance to royal roads
 
 global D E
 D= Dd_clean;                           %D=distance matrix
 E= Drv_clean;                          %E=variance elevation matrix

 global X X_d pop hpop V

 %Choose set of controls
   
    X = X_clean(:,28:end-6);                                                %matrix of controls
    
    V = [ones(size(p,1),1) zscore(X_clean(:,end-5)) X_clean(:,end-4:end-2)];                         %matrix of covariates for the unobserved national weights
    
    hpop = X_clean(:,end-1:end);                                            %matrix of 1843 population and missing 1843 population dummy
    X_d = X_clean(:,2:27);                                                  %matrix of department dummies and department capital dummy
    pop = X_clean(:,1)./1000;                                               %vector of population
 
 global A
 A=A_clean;                             %A=adjacency matrix (nxn) with 0s on the diagonal

 global n k ndep
 n=size(p,1);                           %number of observations
 k=size(X,2);                           %number of covariates
 ndep=size(X_d,2);                      %number of department dummies
 
 DaneCodes=DaneCodes_clean;             %Municipality codes to keep track
 
 %Create Indicator of violent municipalities (above 90th percentile of attacks)
 v=zeros(n,1);
 pc90 = prctile(Yv_clean,90);
 for j=1:n
     if Yv_clean(j,1)>=pc90
         v(j)=1;
     end
 end
 
 clear Yg_clean S_clean C_clean X_clean R_clean Yv_clean A_clean Dd_clean Drr_clean Drv_clean DaneCodes_clean
 
 
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Assumptions on the Network Structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if excl_viol==2
    p = p(~v,:);
    l = l(~v,:);
    c = c(~v,:);
    r = r(~v,:);
    X = X(~v,:);
    X_d =X_d(~v,:);
    pop = pop(~v,:);
    hpop = hpop(~v,:);
    V = V(~v,:); 
    
    D0 = D(~v,:)';
    D = (D0(~v,:))';
    
    E0 = E(~v,:)';
    E = (E0(~v,:))';
    
    A0 = A(~v,:)';
    A = (A0(~v,:))';

    clear D0 E0 A0

    n=size(p,1);
end

global NE N N_2 A_2 A_3
F=1./(1+(D.*(1+E)));

%Re-sets the diagonal to zeroes
for i=1:size(F,1)
    F(i,i)=0;
end

Q=(ones(n,n)-A).*(A*A-diag(diag(A*A)));                 %Matrix of Number of Paths to Neighbors of Neighbors who are not own Neighbors
A_2=ismember(Q,1:1:max(max(Q)));                        %Adjacency Matrix of neighbors of neighbors
A_2=+A_2;                                               % from logical to numeric

Q2=(ones(n,n)-A-A_2).*((A*A*A)-diag(diag(A*A*A)));      %Matrix of Number of Paths to Neighbors of Neighbors of Neighbors who are not own Neighbors or own Neighbors of Neighbors
A_3 = ismember(Q2,1:1:max(max(Q2)));                    %Adjacency Matrix of neighbors of neighbors of neighbors
A_3=+A_3;                                               % from logical to numeric
clear Q Q2


if ass_network_structure==1 
    A=(A+A_2);                                          %Adjacency matrix when Network link is defined by assumption b
    A_2=A_3;                                            %Adjacency matrix of neighbors of neighbors when Network link is defined by assumption b
end

%Network matrix of adjacent neighbors
NE = F.*A; 

if order==2
    N=F.*A;                                                 %Network matrix of neighbors
    N_2=F.*A_2;                                             %Network matrix of neighbors of neighbors
else if order==3
        N=F.*A_2;                                           %Network matrix of neighbors of neighbors
        N_2=F.*A_3;                                         %Network matrix of neighbors of neighbors of neighbors
    end
end
 
 %%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %SIMULATED GMM
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% %Municipality-Level FOC
% %l = f( Ns , c1 , c2, r, , X , X_d)
%
% %Outcomes Equation
% %y = f(s, N*s, X , X_d)
% 
% %National-Level FOC
  %b = f(l, Ns, zet, X, X_d)
  
%%%%%%First stage GMM
 
 assum_f_delta=0;    % if 0 we are using Assumption f(D,E), 1 if we are using assumption f(D,E,delta).
 options=optimset('Display','iter','MaxIter',80000,'MaxFunEvals',80000,'Algorithm','interior-point');
 biweight_kernel=0;                                      %0 for normal kernel, 1 for biweight kernel
 truncation_normal=160;                                  %Truncation for normal kernel: the lower the value, the stronger the truncation
 nobin=1;                                                %0 for using bins , 1 for using directly the grid of observations
 
 global num_out num_eq
 num_out= size(p,2);
 num_eq = num_out+2;  %One equation for each outcome plus the local-level best response plus the national-level best response
 
 num_cov_weights = size(V,2);
 
 global d_bar e_bar
     number_of_neighbors=sum(A,2);
     d_bar=(1/1)*(number_of_neighbors).^(-1).*sum(A.*D,2);
     e_bar=(1/1)*(number_of_neighbors).^(-1).*sum(A.*E,2);

     %Initialize parameter value for delta
      ddelta_guess = 1*ones(1,2);    
      
 if assum_f_delta==1     % if N(d) depends on delta (the neighbors matrices are assumed to depend on a parameter vector delta)
          
      ddelta=ddelta_guess;
      [ N_d, N_d_2, F_d] = neighbors_matrix( ddelta, order );  % call a function that generates the neighbors and neighbors of neighbors matrices which depend on delta
      NN=N_d;
      NN_2=N_d_2;    
     
      Z_out_w=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (NN*r).^3 (NN*log(1+c)).^4 (NN*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop zscore(d_bar) zscore(e_bar)];
      Z_br_w =[NN*log(1+c) NN*r NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2) zscore(d_bar) zscore(e_bar)];
      Z_nat_w=[V zscore(X) X_d log(hpop(:,1)) hpop(:,2) zscore(d_bar) zscore(e_bar)];
      
      
      lb = [0.01  0 -Inf*ones(1,num_out + num_out*(k+1+ndep) + 3 + k + 1 + ndep) 0.11  0.9  0.5    0 -Inf*ones(1,num_cov_weights)   0    0]';
      ub = [  10 10  Inf*ones(1,num_out + num_out*(k+1+ndep) + 3 + k + 1 + ndep) 0.11  0.9  0.5  Inf  Inf*ones(1,num_cov_weights) Inf  Inf]';    
 else
     NN=N;
     NN_2=N_2;
          
     Z_out_w=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop];
     Z_br_w =[NN*log(1+c) NN*r NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2)];
     Z_nat_w=[V zscore(X) X_d log(hpop(:,1)) hpop(:,2)];            
       
     lb = [0.01  0 -Inf*ones(1,num_out + num_out*(k+1+ndep) + 3 + k + 1 + ndep) 0.11  0.9  0.5    0 -Inf*ones(1,num_cov_weights)]';
     ub = [  10 10  Inf*ones(1,num_out + num_out*(k+1+ndep) + 3 + k + 1 + ndep) 0.11  0.9  0.5  Inf  Inf*ones(1,num_cov_weights)]';  
 end
 
 num_mom = (num_out*size(Z_out_w,2))+size(Z_br_w,2)+size(Z_nat_w,2); %total number of moments
 
 %Instruments for the Weighting  Matrix
    Zw = zeros(num_eq,num_mom,n);
    for obs_n=1:n
        Zw(:,:,obs_n) = [kron(eye(num_out),Z_out_w(obs_n,:)) zeros(num_out,size(Z_br_w,2)) zeros(num_out,size(Z_nat_w,2)); zeros(1,(num_out*size(Z_out_w,2))) Z_br_w(obs_n,:) zeros(1,size(Z_nat_w,2)); zeros(1,(num_out*size(Z_out_w,2))) zeros(1,size(Z_br_w,2)) Z_nat_w(obs_n,:)];
    end  
  
    ZOZ=zeros(num_mom,num_mom);
 
    for i=1:n
        ZOZ = Zw(:,:,i)'*Zw(:,:,i) + ZOZ;
    end

    %Omegas to integrate out in the simulation:
    %standard normal draws
    draws=100;
    myseed=10;
    rng(myseed);
    OMEGAS = randn(n,draws);
      
    %Initialize parameter values
      
      theta_guess = 0.024;      
      psi_1_guess = 0.006;
      psi_2_guess = [1.21 0.93 0.74 0.87];%*ones(1,num_out);
      beta_out_guess = 0*ones(1,num_out*(k+1));
      zet_out_guess = 0*ones(1,num_out*ndep);
      phi_guess = 0.01*ones(1,3);
      beta_br_guess = 0.01*ones(1,k+1);
      zet_br_guess = 0.01*ones(1,ndep);
      sigma_guess = 0.11;
      a_guess = 0.9;
      tau_guess = 0.5;
      eta_guess = 0.5;
      pis_guess =0.1*ones(1,num_cov_weights);
      
      if assum_f_delta==1
          guess_gmm = [theta_guess psi_1_guess psi_2_guess beta_out_guess zet_out_guess phi_guess beta_br_guess zet_br_guess sigma_guess a_guess tau_guess eta_guess pis_guess ddelta_guess];   
          tic
            [param_gmm_first, fval_first] = fmincon(@(param_gmm) simul_gmm_objective_function(param_gmm, assum_f_delta, truncation_normal, nobin, biweight_kernel, order, num_out, ZOZ, OMEGAS, num_cov_weights, draws ), guess_gmm, [], [], [], [], lb, ub, [], options);
          toc
      
      else
          guess_gmm = [theta_guess psi_1_guess psi_2_guess beta_out_guess zet_out_guess phi_guess beta_br_guess zet_br_guess sigma_guess a_guess tau_guess eta_guess pis_guess];
          tic
            [param_gmm_first, fval_first] = fmincon(@(param_gmm) simul_gmm_objective_function(param_gmm, assum_f_delta, truncation_normal, nobin, biweight_kernel, order, num_out, ZOZ, OMEGAS, num_cov_weights, draws ), guess_gmm, [], [], [], [], lb, ub, [], options);
          toc
      end
  
 
 %%%%%%Second stage (efficient) System GMM
      
      theta = param_gmm_first(1);
      psi_1 = param_gmm_first(2);
      psi_2 = param_gmm_first(3:3+num_out-1);
      beta_out = param_gmm_first(3+num_out:3+num_out+(num_out*(k+1))-1)';
      zet_out = param_gmm_first(3+num_out+(num_out*(k+1)):3+num_out+(num_out*(k+1))+(num_out*ndep)-1)';
      phi = param_gmm_first(3+num_out+(num_out*(k+1))+(num_out*ndep):3+num_out+(num_out*(k+1))+(num_out*ndep)+3-1)';
      beta_br = param_gmm_first(3+num_out+(num_out*(k+1))+(num_out*ndep)+3:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1-1)';
      zet_br = param_gmm_first(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1)';
      sigma = param_gmm_first(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1+1);
      alpha = param_gmm_first(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1+1+1);
      tau = param_gmm_first(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1+1+1+1);
      eta = param_gmm_first(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1+1+1+1+1);
      pis =param_gmm_first(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1+1+1+1+1+1:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1+1+1+1+1+num_cov_weights-1+1);
disp('theta psi_1')
disp([theta psi_1])
      
if assum_f_delta==1                                            % if N(d) depends on delta (the neighbors matrices are assumed to depend on a parameter vector delta)      
      ddelta = param_gmm_first(end-1:end);   
      [ N_d, N_d_2, F_d] = neighbors_matrix( ddelta, order );  % call a function that generates the neighbors and neighbors of neighbors matrices which depend on delta
      NN=N_d;
      NN_2=N_d_2;
      
      Z_out_w=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop zscore(d_bar) zscore(e_bar)];
      Z_br_w =[NN*log(1+c) NN*r NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2) zscore(d_bar) zscore(e_bar)];
      Z_nat_w=[V zscore(X) X_d log(hpop(:,1)) hpop(:,2) zscore(d_bar) zscore(e_bar)];
      
else
     NN=N;
     NN_2=N_2;
     
     Z_out_w=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop];
     Z_br_w =[NN*log(1+c) NN*r NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2)];
     Z_nat_w=[V zscore(X) X_d log(hpop(:,1)) hpop(:,2)];  
end      

%State Capacity Composite
if sigma==1
        s = (l.^alpha).*((tau*b).^(1-alpha));
    else
        s = ( (alpha.*(l.^((sigma-1)/sigma))) + ((1-alpha).*((tau*b).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end
    
%Non parametric conditional expectation
    pphi=[phi; beta_br];
    xx=[log(1+c) r zscore(X) log(pop)];    
    exp_cond_g= nonparametric_conditional( (theta./alpha).*l.*((l./s).^(1/sigma)) - psi_1*(NN*s) - X_d*zet_br, xx, biweight_kernel, truncation_normal, pphi, nobin );  % It gives the conditional expectation of s+ppsi_gmm*N*s-X*bbeta_gmm conditional on c and lambda for each bin

    %Residuals Prosperity:
    Qt = zeros(n,num_out);
    for i=1:num_out
        Qt(:,i) = zscore(p(:,i)) - ((theta./alpha).*l.*s.*((l./s).^(1/sigma))) - (psi_2(i).*(NN*s)) - ([zscore(X) pop]*beta_out(((i-1)*k)+i:i*(k+1))) - (X_d*zet_out((1+((i*ndep)-ndep):i*ndep))); 
    end
    
        %Residuals Best Response Local 
        q_br = (theta./alpha).*l.*((l./s).^(1/sigma)) - psi_1*(NN*s) - exp_cond_g - X_d*zet_br;
        
        %Residuals Best Response National
    
        LS = ((l./s).^(1/sigma));
        
        %SB = ((s./b).^(1/sigma));
        
        q_nat=zeros(n,draws);
        
        for i=1:draws
            q_nat(:,i) = q_nat_moment(alpha, tau, sigma, theta, psi_1, psi_2, eta, num_out, s, LS, NN, exp(V*pis' + OMEGAS(:,i)) );
        end
        
        q_nat_avg =sum(q_nat,2)/draws;
        
        Qt = [Qt q_br q_nat_avg];
        
        Q=Qt';
        
    %Residuals for the Weighting Matrix       
    OM=zeros(num_eq,num_eq,n);
    for obs_n=1:n
        OM(:,:,obs_n) =Q(:,obs_n)*Q(:,obs_n)';
    end
 
  
    %Instruments for the Weighting  Matrix
        Zw = zeros(num_eq,num_mom,n);
        for obs_n=1:n
            Zw(:,:,obs_n) = [kron(eye(num_out),Z_out_w(obs_n,:)) zeros(num_out,size(Z_br_w,2)) zeros(num_out,size(Z_nat_w,2)); zeros(1,(num_out*size(Z_out_w,2))) Z_br_w(obs_n,:) zeros(1,size(Z_nat_w,2)); zeros(1,(num_out*size(Z_out_w,2))) zeros(1,size(Z_br_w,2)) Z_nat_w(obs_n,:)];
        end  
        
    ZOZ=zeros(num_mom,num_mom);
    for i=1:n
        ZOZ = Zw(:,:,i)'*OM(:,:,i)*Zw(:,:,i) + ZOZ;
    end
   
 guess_gmm = param_gmm_first;
 options=optimset('Display','iter','MaxIter',200000,'MaxFunEvals',200000,'Algorithm','interior-point');
         tic
         [param_gmm_final, fval_final] = fmincon(@(param_gmm) simul_gmm_objective_function(param_gmm, assum_f_delta, truncation_normal, nobin, biweight_kernel, order, num_out, ZOZ, OMEGAS, num_cov_weights, draws ), guess_gmm, [], [], [], [], lb, ub, [], options);
         toc
 
         
      theta = param_gmm_final(1);
      psi_1 = param_gmm_final(2);
      psi_2 = param_gmm_final(3:3+num_out-1);
      beta_out = param_gmm_final(3+num_out:3+num_out+(num_out*(k+1))-1)';
      zet_out = param_gmm_final(3+num_out+(num_out*(k+1)):3+num_out+(num_out*(k+1))+(num_out*ndep)-1)';
      phi = param_gmm_final(3+num_out+(num_out*(k+1))+(num_out*ndep):3+num_out+(num_out*(k+1))+(num_out*ndep)+3-1)';
      beta_br = param_gmm_final(3+num_out+(num_out*(k+1))+(num_out*ndep)+3:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1-1)';
      zet_br = param_gmm_final(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1)';
      sigma = param_gmm_final(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1+1);
      alpha = param_gmm_final(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1+1+1);
      tau = param_gmm_final(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1+1+1+1);
      eta = param_gmm_final(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1+1+1+1+1);
      pis =param_gmm_final(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1+1+1+1+1+1:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1+1+1+1+1+num_cov_weights-1+1);
                                    
      
 if assum_f_delta==1                                            % if N(d) depends on delta (the neighbors matrices are assumed to depend on a parameter vector delta)      
      ddelta = param_gmm_final(end-1:end); 
      [ N_d, N_d_2, F_d] = neighbors_matrix( ddelta, order );  % call a function that generates the neighbors and neighbors of neighbors matrices which depend on delta
      NN=N_d;
      NN_2=N_d_2; 
 else
     NN=N;
     NN_2=N_2;
 end      

 %State Capacity Composite
if sigma==1
        s = (l.^alpha).*((tau*b).^(1-alpha));
    else
        s = ( (alpha.*(l.^((sigma-1)/sigma))) + ((1-alpha).*((tau*b).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end
 
disp('parameters estimates for System GMM')
disp('theta')
disp(theta)
disp('psi1')
disp(psi_1)
disp('psi_2')
disp(psi_2)

disp('sigma')
disp(sigma)
disp('alpha')
disp(alpha)
disp('eta')
disp(eta/tau)
disp('PIs')
disp(pis')

if assum_f_delta==1
    disp('delta')
    disp(ddelta')
end
    
    %Non parametric conditional expectation
    yy = (theta./alpha).*l.*((l./s).^(1/sigma)) - psi_1*(NN*s) - X_d*zet_br;
    xx = [log(1+c) r zscore(X) log(pop)];
    exp_cond_g = nonparametric_conditional( yy, xx, biweight_kernel, truncation_normal, [phi; beta_br], nobin );
    
kappa = theta.*exp_cond_g;
E_kappa = theta*mean(exp_cond_g);
disp('E[kappa]')
disp(E_kappa)

stdev_kappa = theta*std(exp_cond_g);
disp('Stdev Kappa')
disp(stdev_kappa)

zetas = exp(V*pis');
disp('Average zetas')
disp(mean(zetas))

disp('Median zetas')
disp(median(zetas))

disp('Standard Deviation zetas')
disp(std(zetas))

%%
%%%% STANDARD ERRORS %%%%
 
    if assum_f_delta==1     %if 0 we are using Assumption f(D,E), 1 if we are using assumption f(D,E,delta).   
     DF(:,:,1)=-D.*(1+ddelta(2)*E).*(F_d).^(-2);
     DN(:,:,1)=A.*DF(:,:,1);
     DN_2(:,:,1)=A_2.*DF(:,:,1);
     DF(:,:,2)=-ddelta(1)*D.*E.*(F_d).^(-2);
     DN(:,:,2)=A.*DF(:,:,2);
     DN_2(:,:,2)=A_2.*DF(:,:,2);
     DZ_out_del(:,:,1)=[DN(:,:,1)*log(1+c) DN(:,:,1)*r (DN(:,:,1)*log(1+c)).^2 (DN(:,:,1)*r).^2 (DN(:,:,1)*log(1+c)).^3 (DN(:,:,1)*r).^3 (DN(:,:,1)*log(1+c)).^4 (DN(:,:,1)*r).^4 DN_2(:,:,1)*log(1+c) DN_2(:,:,1)*r (DN_2(:,:,1)*log(1+c)).^2 (DN_2(:,:,1)*r).^2 (DN_2(:,:,1)*log(1+c)).^3 (DN_2(:,:,1)*r).^3 (DN_2(:,:,1)*log(1+c)).^4 (DN_2(:,:,1)*r).^4];
     DZ_out_del(:,:,2)=[DN(:,:,2)*log(1+c) DN(:,:,2)*r (DN(:,:,2)*log(1+c)).^2 (DN(:,:,2)*r).^2 (DN(:,:,2)*log(1+c)).^3 (DN(:,:,2)*r).^3 (DN(:,:,2)*log(1+c)).^4 (DN(:,:,2)*r).^4 DN_2(:,:,2)*log(1+c) DN_2(:,:,2)*r (DN_2(:,:,2)*log(1+c)).^2 (DN_2(:,:,2)*r).^2 (DN_2(:,:,2)*log(1+c)).^3 (DN_2(:,:,2)*r).^3 (DN_2(:,:,2)*log(1+c)).^4 (DN_2(:,:,2)*r).^4];
     DZ_br_del(:,:,1)=[DN(:,:,1)*log(1+c) DN(:,:,1)*r DN_2(:,:,1)*log(1+c) DN_2(:,:,1)*r];
     DZ_br_del(:,:,2)=[DN(:,:,2)*log(1+c) DN(:,:,2)*r DN_2(:,:,2)*log(1+c) DN_2(:,:,2)*r];
     
      Z_out=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop zscore(d_bar) zscore(e_bar)];
      Z_br =[NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2) zscore(d_bar) zscore(e_bar)];
      Z_nat=[V zscore(X) X_d log(hpop(:,1)) hpop(:,2) zscore(d_bar) zscore(e_bar)];
    else
        Z_out=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop];
        Z_br =[NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2)];
        Z_nat=[V zscore(X) X_d log(hpop(:,1)) hpop(:,2)];  
    end
 
    num_par=size(param_gmm_final,2);
    mom_br =size(Z_br,2);
    mom_out=size(Z_out,2);
    mom_nat=size(Z_nat,2);
    
%Construction of Z_Dq_Arr

    Zi_Arr = zeros(num_eq,num_mom,n);
        for obs_n=1:n
            Zi_Arr(:,:,obs_n) = [kron(eye(num_out),Z_out_w(obs_n,:)) zeros(num_out,size(Z_br_w,2)) zeros(num_out,size(Z_nat_w,2)); zeros(1,(num_out*size(Z_out_w,2))) Z_br_w(obs_n,:) zeros(1,size(Z_nat_w,2)); zeros(1,(num_out*size(Z_out_w,2))) zeros(1,size(Z_br_w,2)) Z_nat_w(obs_n,:)];
        end    
    
   %Non parametric conditional expectation
    pphi=[phi; beta_br];
    xx=[log(1+c) r zscore(X) log(pop)];    
    exp_cond_g= nonparametric_conditional( (theta./alpha).*l.*((l./s).^(1/sigma)) - psi_1*(NN*s) - X_d*zet_br, xx, biweight_kernel, truncation_normal, pphi, nobin );  % It gives the conditional expectation of s+ppsi_gmm*N*s-X*bbeta_gmm conditional on c and lambda for each bin

    %Residuals Prosperity:
    Qt = zeros(n,num_out);
    for i=1:num_out
        Qt(:,i) = zscore(p(:,i)) - ((theta./alpha).*l.*s.*((l./s).^(1/sigma))) - (psi_2(i).*(NN*s)) - ([zscore(X) pop]*beta_out(((i-1)*k)+i:i*(k+1))) - (X_d*zet_out((1+((i*ndep)-ndep):i*ndep))); 
    end
    %Residuals Best Response Local 
        q_br = (theta./alpha).*l.*((l./s).^(1/sigma)) - psi_1*(NN*s) - exp_cond_g - X_d*zet_br;
   
     %Residuals Best Response National
    
        LS = ((l./s).^(1/sigma));
        
        SB = ((s./b).^(1/sigma));
        
        q_nat=zeros(n,draws);
        
        for i=1:draws
            q_nat(:,i) =q_nat_moment(alpha, tau, sigma, theta, psi_1, psi_2, eta, num_out, s, LS, NN, exp(V*pis' + OMEGAS(:,i)) );
        end
        
        q_nat_avg =sum(q_nat,2)/draws;
        
        Qt = [Qt q_br q_nat_avg];
        
        Q=Qt';
     
 
    %Residuals for the Weighting Matrix       
    OM=zeros(num_eq,num_eq,n);
    for obs_n=1:n
        OM(:,:,obs_n) =Q(:,obs_n)*Q(:,obs_n)';
    end
    
    ZuuZ=zeros(num_mom,num_mom);
    for i=1:n
        ZuuZ = Zi_Arr(:,:,i)'*OM(:,:,i)*Zi_Arr(:,:,i) + ZuuZ;
    end
    
    
 %Average derivative of g(phi*c + X*b):
 g=[xx*pphi exp_cond_g];
 
 b_g = ([ones(n,1) g(:,1)]'*[ones(n,1) g(:,1)])\([ones(n,1) g(:,1)]'*g(:,2));
 g_prime=b_g(2);
 

 %Construction of Dq   (Gradient of equations)
 
 % derivatives of state capacity over parameters
 deriv_s_over_sigma=((alpha.*l.^((sigma-1)/sigma)-(alpha-1).*(b*tau).^((sigma-1)/sigma)).^(sigma/(sigma-1)).*(-sigma.*(-alpha.*l.*(b*tau).^(1/sigma)...
     +alpha.*(b*tau).*l.^(1/sigma)-(b*tau).*l.^(1/sigma)).*log(alpha.*l.^((sigma-1)/sigma)-(alpha-1).*(b*tau).^((sigma-1)/sigma))-alpha.*l.*(sigma-1).*(b*tau).^(1/sigma).*log(l)...
     +(alpha-1).*(b*tau).*(sigma-1).*log((b*tau)).*l.^(1/sigma)))./((sigma-1)^2 .*sigma.*(-alpha.*l.*(b*tau).^(1/sigma)+alpha.*(b*tau).*l.^(1/sigma)-(b*tau).*l.^(1/sigma)));
 
 deriv_s_over_tau=(1-alpha).*s.^(1/sigma).*b.^((sigma-1)/sigma).*tau.^(-1/sigma);
 
 deriv_s_over_alpha=(sigma/(sigma-1))*s.^(1/sigma).*(l.^((sigma-1)/sigma)-(tau*b).^((sigma-1)/sigma));
 
 Dq=zeros(num_eq,num_par,n);
 
 % Outcomes equations
 for j=1:num_out
     Dq(j,1,:)=(l.*s/alpha).*LS/tau;
     Dq(j,2,:)=zeros(n,1);
     
     if j==1
         Dq(j,3,:)=-NN*s;
         Dq(j,4,:)=zeros(n,1);
         Dq(j,5,:)=zeros(n,1);
         Dq(j,6,:)=zeros(n,1);
     else if j==2
             Dq(j,3,:)=zeros(n,1);
             Dq(j,4,:)=-NN*s;
             Dq(j,5,:)=zeros(n,1);
             Dq(j,6,:)=zeros(n,1);
         else if j==3
                 Dq(j,3,:)=zeros(n,1);
                 Dq(j,4,:)=zeros(n,1);
                 Dq(j,5,:)=-NN*s;
                 Dq(j,6,:)=zeros(n,1);
             else if j==4
                     Dq(j,3,:)=zeros(n,1);
                     Dq(j,4,:)=zeros(n,1);
                     Dq(j,5,:)=zeros(n,1);
                     Dq(j,6,:)=-NN*s;
                 end
             end
         end
     end
     
     Dq(j,7+((j-1)*(k+1)):7+(j*(k+1))-1,:)=-[zscore(X) pop]';
     Dq(j,7+(num_out*(k+1))+((j-1)*ndep):7+(num_out*(k+1))+(j*ndep)-1,:)=-X_d';
     Dq(j,7+(num_out*(k+1))+(num_out*ndep):7+(num_out*(k+1))+(num_out*ndep)+3-1,:)=zeros(n,3)';
     Dq(j,7+(num_out*(k+1))+(num_out*ndep)+3:7+(num_out*(k+1))+(num_out*ndep)+3+k+1-1,:)=zeros(n,k+1)';
     Dq(j,7+(num_out*(k+1))+(num_out*ndep)+3+k+1:7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1,:)=zeros(n,ndep)';
     
     Dq(j,7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep,:)=-(theta./alpha).*(l.^((sigma+1)/sigma)).*s.^((sigma-1)/sigma).*(sigma^(-2)*log(s)+((sigma-1)/sigma)*s.^(-1).*deriv_s_over_sigma)-psi_2(j)*NN*deriv_s_over_sigma;
     
     Dq(j,7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+1,:)=(theta/(alpha^2))*((l.^(1+(1/sigma))).*((tau*b).^((sigma-1)/sigma)));
     Dq(j,7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+2,:)=-(theta./alpha).*l.^((1+sigma)/sigma).*(-(1/sigma)*s.^(-(1+sigma)/sigma)).*deriv_s_over_tau;
     Dq(j,7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+3,:)=zeros(n,1)';
     Dq(j,7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+4:7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+3+num_cov_weights,:)=zeros(n,num_cov_weights)';
     
     if assum_f_delta==1
         Dq(j,end-1,:)=-psi_2(j)*(DN(:,:,1)*s);
         Dq(j,end,:)=-psi_2(j)*(DN(:,:,2)*s);
     end
 end
 
 % Best response equation
 
 Dq(num_out+1,1,:)=(l./alpha).*LS/tau;
 Dq(num_out+1,2,:)=-(1/psi_1)*(NN*s);
 Dq(num_out+1,3,:)=zeros(n,1);
 Dq(num_out+1,4,:)=zeros(n,1);
 Dq(num_out+1,5,:)=zeros(n,1);
 Dq(num_out+1,6,:)=zeros(n,1);
 Dq(num_out+1,7:7+(num_out*(k+1))-1,:)=zeros(n,num_out*(k+1))';
 Dq(num_out+1,7+(num_out*(k+1)):7+(num_out*(k+1))+(num_out*ndep)-1,:)=zeros(n,num_out*ndep)';
 Dq(num_out+1,7+(num_out*(k+1))+(num_out*ndep):7+(num_out*(k+1))+(num_out*ndep)+3-1,:)=-xx(:,1:3)'*g_prime;
 Dq(num_out+1,7+(num_out*(k+1))+(num_out*ndep)+3:7+(num_out*(k+1))+(num_out*ndep)+3+k+1-1,:)=-[zscore(X) log(pop)]'*g_prime;
 Dq(num_out+1,7+(num_out*(k+1))+(num_out*ndep)+3+k+1:7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1,:)=-X_d';
 
 Dq(num_out+1,7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep,:)=(theta./alpha).*(l.^((sigma+1)/sigma)).*s.^(-1/sigma).*(sigma^(-2)*log(s)-(sigma*s).^(-1).*deriv_s_over_sigma)-psi_1*NN*deriv_s_over_sigma;
 Dq(num_out+1,7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+1,:)=(theta/alpha)*l.*((l./s).^(1/sigma)).*( ((1/(1-sigma)).*(s.^(1+(1/sigma))).*( l.^((sigma-1)/sigma) - ((tau*b).^((sigma-1)/sigma)) ))   - (1/(alpha^2)) );
 Dq(num_out+1,7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+2,:)=(theta./alpha).*(l.^((sigma+1)/sigma)).*(-(1/sigma)*s.^(-(1+sigma)/sigma)).*deriv_s_over_tau;
 Dq(num_out+1,7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+3,:)=zeros(n,1)';
 Dq(num_out+1,7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+4:7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+3+num_cov_weights,:)=zeros(n,num_cov_weights)';
 
 if assum_f_delta==1
     Dq(num_out+1,end-1,:)=-psi_1*((DN(:,:,1)*s));
     Dq(num_out+1,end,:)=-psi_1*((DN(:,:,2)*s));
 end
 
 % National equation
 
 Dq_nat=zeros(num_par,n,draws);
 
 for i=1:draws
 ZET=1+exp(V*pis' + OMEGAS(:,i));
 Dq_nat(1,:,i) = ((1-alpha)/alpha)*tau^((sigma-1)/sigma)*(s./b).^(1/sigma).*l.*LS.*ZET;
 Dq_nat(2,:,i) =  (1-alpha)*tau^((sigma-1)/sigma)*(s./b).^(1/sigma).*(NN*(s.*ZET));
 Dq_nat(3,:,i) = (1-alpha)*tau^((sigma-1)/sigma)*(s./b).^(1/sigma).*(NN*((ones(n,1)/num_out).*ZET));
 Dq_nat(4,:,i) = (1-alpha)*tau^((sigma-1)/sigma)*(s./b).^(1/sigma).*(NN*((ones(n,1)/num_out).*ZET));
 Dq_nat(5,:,i) = (1-alpha)*tau^((sigma-1)/sigma)*(s./b).^(1/sigma).*(NN*((ones(n,1)/num_out).*ZET));
 Dq_nat(6,:,i) = (1-alpha)*tau^((sigma-1)/sigma)*(s./b).^(1/sigma).*(NN*((ones(n,1)/num_out).*ZET));
 
 Dq_nat(7:7+(num_out*(k+1))-1,:,i) =zeros(n,num_out*(k+1))';
 Dq_nat(7+(num_out*(k+1)):7+(num_out*(k+1))+(num_out*ndep)-1,:,i) =zeros(n,num_out*ndep)';
 Dq_nat(7+(num_out*(k+1))+(num_out*ndep):7+(num_out*(k+1))+(num_out*ndep)+3-1,:,i)=zeros(n,3)';
 Dq_nat(7+(num_out*(k+1))+(num_out*ndep)+3:7+(num_out*(k+1))+(num_out*ndep)+3+k+1-1,:,i)=zeros(n,k+1)';
 Dq_nat(7+(num_out*(k+1))+(num_out*ndep)+3+k+1:7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1,:,i)=zeros(n,ndep)';
 
 Dq_nat(7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep,:,i)= (1-alpha)*tau^((sigma-1)/sigma)*(log(tau)*sigma^(-2)*(s./b).^(1/sigma)+s.^(1/sigma).*...
     (-sigma^(-2)*log(s)+(sigma*s).^(-1).*deriv_s_over_sigma)).*((theta/alpha)*l.*LS.*ZET+NN*((psi_1*s+(sum(psi_2)/num_out)*ones(n,1)).*ZET))+...
     (1-alpha)*tau^((sigma-1)/sigma)*(s./b).^(1/sigma).*((theta/alpha)*ZET.*l.^((1+sigma)/sigma).*s.^(-1/sigma).*(sigma^(-2)*log(s)-(sigma*s).^(-1).*deriv_s_over_sigma));
 Dq_nat(7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+1,:,i) = tau^((sigma-1)/sigma)*b.^(-1/sigma).*((-s.^(1/sigma)+(1-alpha)*sigma^(-1)*s.^((1-sigma)/sigma).*deriv_s_over_alpha).*...
     ((theta/alpha)*l.*LS.*ZET+NN*((psi_1*s+(sum(psi_2)/num_out)*ones(n,1)).*ZET))+(1-alpha)*s.*(1/sigma).*(theta*s.*l.^((1+sigma)/sigma).*(-alpha^(-2)*s.^(-1/sigma)-...
     (alpha*sigma)^(-1)*s.^(-(1+sigma)/sigma).*deriv_s_over_alpha)+NN*(psi_1*ZET.*deriv_s_over_alpha)));
 Dq_nat(7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+2,:,i) = (1-alpha)*b.^(-1/sigma).*((((sigma-1)/sigma)*tau^(-1/sigma).*s.^(1/sigma)+tau^((sigma-1)/sigma)*sigma^(-1)...
     .*s.^((1-sigma)/sigma).*deriv_s_over_tau).*((theta/alpha)*l.*LS.*ZET+NN*((psi_1*s+(sum(psi_2)/num_out)*ones(n,1)).*ZET))+tau^((sigma-1)/sigma)*s.^(1/sigma)...
     .*((theta/alpha)*ZET.*l.^((1+sigma)/sigma).*(-(1/sigma)*s.^(-(1+sigma)/sigma).*deriv_s_over_tau)+NN*(psi_1*ZET.*deriv_s_over_tau)));
 Dq_nat(7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+3,:,i)= -b/tau;

 for ww=1:num_cov_weights
    Dq_nat(7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+3+ww,:,i)= (s./b).^(1/sigma).*((theta/alpha)*l.*LS.^(1/sigma).*ZET.*V(:,ww)+NN*((psi_1*s+(sum(psi_2)/num_out)*ones(n,1)).*ZET.*V(:,ww)));
 end
 
 if assum_f_delta==1
     Dq_nat(end-1,:,i)= (1-alpha)*tau^((sigma-1)/sigma)*(s./b).^(1/sigma).*DN(:,:,1)*((psi_1*s+(sum(psi_2)/num_out)*ones(n,1)).*ZET);
     Dq_nat(end,:,i)= (1-alpha)*tau^((sigma-1)/sigma)*(s./b).^(1/sigma).*DN(:,:,2)*((psi_1*s+(sum(psi_2)/num_out)*ones(n,1)).*ZET);
 end
 end
 
 Dq_nat=sum(Dq_nat,3)/draws;
 
 Dq(num_out+2,:,:)=Dq_nat;
 
 
 Z_Dq_Arr=zeros(num_mom,num_par,n);
 for i=1:n
     Z_Dq_Arr(:,:,i) = Zi_Arr(:,:,i)'*Dq(:,:,i);
 end
    
   
%Construction of DZ_q_Arr

    %Construction of DZ (Gradient of Instruments)
    DZout=zeros(mom_out,num_par,n);
    if assum_f_delta==1
        for i=1:size(DZ_out_del,2)
            DZout(i,end-1,:)= DZ_out_del(:,i,1);
        end
        for i=1:size(DZ_out_del,2)
            DZout(i,end,:)= DZ_out_del(:,i,2);
        end
    end
        
    DZbr=zeros(mom_br,num_par,n);
    if assum_f_delta==1    
        for i=1:size(DZ_br_del,2) 
            DZbr(i,end-1,:)= DZ_br_del(:,i,1);
        end 
        for i=1:size(DZ_br_del,2)
            DZbr(i,end,:)= DZ_br_del(:,i,2);
        end
    end

    %Notice that the set of instruments for the national best response does not depend on delta
    
    DZ_q_Arr=zeros(num_mom,num_par,n);
    if assum_f_delta==1
    for i=1:n
        DZ_q_Arr(:,:,i)=[repmat(DZout(:,:,i),num_out,1); DZbr(:,:,i)].*(repmat([reshape(repmat(Q(1:num_out,i)',mom_out,1),mom_out*num_out,1) ; Q(end,i)*ones(mom_br,1)],1,num_par));
    end
    end
    
 %Construction of DG
 DG_Arr = Z_Dq_Arr + DZ_q_Arr;   
    
 DG = sum(DG_Arr,3);
 
 %% Variance Covariance Matrix of the GMM estimator
 
 % "CAMERON AND TRIVEDI" term
 Var_draws=var(q_nat_avg)/n;
 Exp_var_draws=sum(Var_draws)/draws;
 
 Avar_gmm = inv((DG'/ZuuZ)*DG)+Exp_var_draws;   
 
 std_errors_gmm = sqrt(diag(Avar_gmm));
 disp('Standar Errors for Parameter Estimates System GMM')
 disp('standard error theta')
 disp(std_errors_gmm(1)')
 disp('standard error psi_1')
 disp(std_errors_gmm(2)')
 disp('standard errors psi_2')
 disp(std_errors_gmm(3:6)')

 disp('standard error eta')
 disp(std_errors_gmm(7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+3)')
 disp('standard error pis')
 disp(std_errors_gmm(7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+4:7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep+3+num_cov_weights)')

 if assum_f_delta==1
  disp('standard errors delta')
  disp(std_errors_gmm(end-1:end)')
 end


%%    
%Condition for Equilibrium Uniqueness
lambda_min = min(eig(NN));
disp('Lowest Eigenvalue of the Network Matrix')
disp(lambda_min)

TT = ((sigma+1)*((l./s).^(1/sigma))) - (alpha*(l./s)) ;

dl_dNs = TT./(psi_1*alpha*sigma./theta);

dl_dNs_inv=(dl_dNs).^(-1);

disp('Min (dl_dN(d)s)^(-1)')
disp(min(dl_dNs_inv))

disp('Max (dl_dN(d)s)^(-1)')
disp(max(dl_dNs_inv))

if ( min(dl_dNs_inv)>(1+(1/lambda_min)) ) && ( max(dl_dNs_inv)<1 )
     disp('Unique Equilibrium Condition Satisfied')
 else
     disp('Unique Equilibrium Condition Not Satisfied')
end
     
 

%%
%Marginal Effects

%dL_i/dL_j in the Best Response
Num = ((s./l).^(1/sigma))/tau;
Num = repmat(Num,1,n);

Den = ((sigma+1)*((l./s).^(1/sigma))) - (alpha*(l./s));
Den = repmat(Den,1,n);

Quot = Num./Den;

NQ = NN.*Quot;

Avg_NQ = sum(sum(NQ))./sum(sum(A));

d_li_d_lj = (psi_1*alpha*sigma/theta)*Avg_NQ;

disp('Average Non-linear Marginal effect')
disp(d_li_d_lj)

%dp_i/dl_i in the prosperity equation

d_pi_d_li = 2*theta.*l/tau + theta*((1-alpha)/alpha)*((sigma+1)/sigma).*(l.^(1/sigma)).*((tau.*b).^((sigma-1)/sigma))/tau;

d_pi_d_li = sum(d_pi_d_li)./n;

disp('Average Non-linear Marginal effect')
disp(d_pi_d_li)


%dp_i/dl_j in the prosperity equation
PR = NN*((s.^(1/(sigma-1))).*(l.^(-1/sigma)))/tau;
PR = PR./(sum(A)'/n);

d_pi_d_lj = PR*psi_2;
d_pi_d_lj = mean(d_pi_d_lj);

disp('Average Non-linear Marginal effect')
disp(d_pi_d_lj)


%%
%Experiments: Experiments consist of finding the zero of the system of non-linear best responses after different kinds of shocks. Use Newton-Raphson to find the roots
%% 

%Baseline
l_0 = l.*ones(n,1);
b_0 = b.*ones(n,1);
s_0 = [l_0 b_0];
h_prime_b=0;
ZETA_sh=zetas;

f = @(x) BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g, zet_br, NN, ZETA_sh, h_prime_b, x) ; % function of dummy variable x

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g, zet_br, NN, zetas, h_prime_b, log(s_0))

options = optimset('Display','iter', 'MaxFunEvals', 1000000, 'MaxIter', 1000);
[root, fval] = fsolve(f, log(s_0), options);

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g, zet_br, NN, ZETA_sh, h_prime_b, root)

lg = exp(root(1:n))';
bg = exp(root(n+1:2*n))';

%State Capacity Composite
if sigma==1
        sg = (lg.^alpha).*((tau*bg).^(1-alpha));
    else
        sg = ( (alpha.*(lg.^((sigma-1)/sigma))) + ((1-alpha).*((tau*bg).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end

l_base = exp(exp(root(1:n)));
b_base = exp(exp(root(n+1:2*n)));

p_base=zeros(n,num_out);
for i=1:num_out

    p_base(:,i) = ((theta./alpha).*lg.*sg.*((lg./sg).^(1/sigma))) + psi_2(i)*NN*sg +  ([zscore(X) pop]*beta_out(((i-1)*k)+i:i*(k+1))) + (X_d*zet_out((1+((i*ndep)-ndep):i*ndep)));
    
    p_base(:,i) = (p_base(:,i)*std(p(:,i))) + mean(p(:,i)); 
    
end


%%
%Shift the local-level best response to the median

l_0 = l.*ones(n,1);
b_0 = b.*ones(n,1);
s_0 = [l_0 b_0];
h_prime_b=0;

a=( ((sigma-1)/sigma).*(1./l_0) - (alpha/sigma).*(l_0.^(-1/sigma)).*(s.^((1-sigma)/sigma)) );
D_g = ((theta./alpha).*(l_0).*((l_0./s).^(1/sigma)).*exp(-a.*l_0)).*(exp(a.*l_0) - exp(a.*median(l_0)));    %Introduces the shock to g(.) that takes l to the median if below

for i=1:n
    if l_0(i)>median(l_0)
        D_g(i)=0;
    end
end


f = @(x) BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g+D_g, zet_br, NN, ZETA_sh, h_prime_b, x) ; % function of dummy variable x

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g+D_g, zet_br, NN, zetas, h_prime_b, log(s_0))

options = optimset('Display','iter', 'MaxFunEvals', 1000000, 'MaxIter', 1000);
[root, fval] = fsolve(f, log(s_0), options);

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g+D_g, zet_br, NN, ZETA_sh, h_prime_b, root)

l_predict = exp(exp(root(1:n)));

b_predict = exp(exp(root(n+1:2*n)));

D_l = (1+((median(exp(l))-median(l_base))/median(exp(l))))*median(exp(l))*(1+(median(l_predict) - median(l_base))/median(l_base));

disp('General equilibrium local state after shock')
disp(D_l)


%Partial Equilibrium
lp = max(l,median(l));

%State Capacity Composite
if sigma==1
        sp = (lp.^alpha).*((tau*b).^(1-alpha));
    else
        sp = ( (alpha.*(lp.^((sigma-1)/sigma))) + ((1-alpha).*((tau*b).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end


D_p_partial=zeros(n,num_out);


for i=1:num_out
    D_p_partial(:,i) = ( (((theta./alpha).*lp.*sp.*((lp./sp).^(1/sigma))) + (psi_2(i).*(NN*sp))) - (((theta./alpha).*l.*s.*((l./s).^(1/sigma))) + (psi_2(i).*(NN*s))) )./(((theta./alpha).*l.*s.*((l./s).^(1/sigma))) + (psi_2(i).*(NN*s)) )   ;
end

D_p_partial_l = (1+D_p_partial).*p;

disp('Partial Equilibrium Outcomes after shock')
disp(median(D_p_partial_l))


%General Equilibrium
lg = (1+(((exp(l))-(l_base'))./(exp(l)))).*log(l_predict)';
lg = max(lg,0);

bg = (1+(((exp(b))-(b_base'))./(exp(b)))).*log(b_predict)';
bg = max(bg,0);

%State Capacity Composite
if sigma==1
        sg = (lg.^alpha).*((tau*bg).^(1-alpha));
    else
        sg = ( (alpha.*(lg.^((sigma-1)/sigma))) + ((1-alpha).*((tau*bg).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end


D_p_gen=zeros(n,num_out);


for i=1:num_out
    D_p_gen(:,i) = ( (((theta./alpha).*lg.*sg.*((lg./sg).^(1/sigma))) + (psi_2(i).*(NN*sg))) - (((theta./alpha).*l.*s.*((l./s).^(1/sigma))) + (psi_2(i).*(NN*s))) )./(((theta./alpha).*l.*s.*((l./s).^(1/sigma))) + (psi_2(i).*(NN*s)) );
end

D_p_gen_l = (1+D_p_gen).*p;
D_p_gen_l_1 = D_p_gen_l(:,1);
D_p_gen_l_2 = D_p_gen_l(:,2);
D_p_gen_l_3 = D_p_gen_l(:,3);
D_p_gen_l_4 = D_p_gen_l(:,4);

D_p_gen_l_1 = D_p_gen_l_1(~isnan(D_p_gen_l_1));
D_p_gen_l_2 = D_p_gen_l_2(~isnan(D_p_gen_l_2));
D_p_gen_l_3 = D_p_gen_l_3(~isnan(D_p_gen_l_3));
D_p_gen_l_4 = D_p_gen_l_4(~isnan(D_p_gen_l_4));

disp('General Equilibrium Outcomes after shock')
disp(median([D_p_gen_l_1 D_p_gen_l_2 D_p_gen_l_3 D_p_gen_l_4]))


%%
%National Level shock: Move all National-level state capacity to the median

l_0 = l.*ones(n,1);
b_0 = b.*ones(n,1);
s_0 = [l_0 b_0];

%Shock to national-level state capacity

h_prime_b = (-1/sigma)*(1-alpha)*(tau^((sigma-1)/sigma))*(  ((theta/alpha)*zetas.*(l./b).*((l.*b).^(1/sigma)))  - ((NN*(((psi_1*s) + ((sum(psi_2)/num_out).*ones(n,1))).*ZETA_sh)).*(( ((1-alpha)*(tau^((sigma-1)/sigma)).*(s.^((2-sigma)/sigma))) - ((b.^((1-sigma)/sigma)).*(s.^(1/sigma))) )./(b.^(2/sigma)))  ) ) - eta;

for i=1:n
    if b_0(i)>median(b_0)
        h_prime_b(i)=0;
    end
end

h_prime_b =h_prime_b./b;


f = @(x) BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g, zet_br, NN, ZETA_sh, h_prime_b, x) ; % function of dummy variable x

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g, zet_br, NN, ZETA_sh, h_prime_b, log(s_0))

options = optimset('Display','iter', 'MaxFunEvals', 1000000, 'MaxIter', 1000);
[root, fval] = fsolve(f, log(s_0), options);

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g, zet_br, NN, ZETA_sh, h_prime_b, root)

l_predict = exp(exp(root(1:n)));

b_predict = exp(exp(root(n+1:2*n)));

D_l = exp(l_0)'.*(1+(l_predict - l_base)./l_base);
D_b = exp(b_0)'.*(1+(b_predict - b_base)./b_base);

disp('General equilibrium local state after shock')
disp(median(D_l))

disp('General equilibrium national state after shock')
disp(tau*median(D_b))


%General Equilibrium
lg = log(l_predict)';

bg = log(b_predict)';


%State Capacity Composite
if sigma==1
        sg = (lg.^alpha).*((tau*bg).^(1-alpha));
    else
        sg = ( (alpha.*(lg.^((sigma-1)/sigma))) + ((1-alpha).*((tau*bg).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end


%General Equilibrium
lb = log(l_base)';

bb = log(b_base)';


%State Capacity Composite
if sigma==1
        sb = (lb.^alpha).*((tau*bb).^(1-alpha));
    else
        sb = ( (alpha.*(lb.^((sigma-1)/sigma))) + ((1-alpha).*((tau*bb).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end


D_p_gen=zeros(n,num_out);


for i=1:num_out
    D_p_gen(:,i) = ( (((theta./alpha).*lg.*sg.*((lg./sg).^(1/sigma))) + (psi_2(i).*(NN*sg))) - (((theta./alpha).*lb.*sb.*((lb./sb).^(1/sigma))) + (psi_2(i).*(NN*sb))) )./(((theta./alpha).*lb.*sb.*((lb./sb).^(1/sigma))) + (psi_2(i).*(NN*sb)));
    
end

D_p_gen_l = (1+D_p_gen).*p;
D_p_gen_l_1 = D_p_gen_l(:,1);
D_p_gen_l_2 = D_p_gen_l(:,2);
D_p_gen_l_3 = D_p_gen_l(:,3);
D_p_gen_l_4 = D_p_gen_l(:,4);

D_p_gen_l_1 = D_p_gen_l_1(~isnan(D_p_gen_l_1));
D_p_gen_l_2 = D_p_gen_l_2(~isnan(D_p_gen_l_2));
D_p_gen_l_3 = D_p_gen_l_3(~isnan(D_p_gen_l_3));
D_p_gen_l_4 = D_p_gen_l_4(~isnan(D_p_gen_l_4));

disp('General Equilibrium Outcomes after shock')
disp(median([D_p_gen_l_1 D_p_gen_l_2 D_p_gen_l_3 D_p_gen_l_4]))




%%
%National Level shock: Move all National-level weights to the median

l_0 = l.*ones(n,1);
b_0 = b.*ones(n,1);
s_0 = [l_0 b_0];
h_prime_b=0;
%Shock to national-level weights
ZETA_sh = max(median(zetas),zetas);


f = @(x) BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g, zet_br, NN, ZETA_sh, h_prime_b, x) ; % function of dummy variable x

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g, zet_br, NN, ZETA_sh, h_prime_b, log(s_0))

options = optimset('Display','iter', 'MaxFunEvals', 1000000, 'MaxIter', 1000);
[root, fval] = fsolve(f, log(s_0), options);

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g, zet_br, NN, ZETA_sh, h_prime_b, root)


l_predict = exp(exp(root(1:n)));

b_predict = exp(exp(root(n+1:2*n)));

D_l = exp(l_0)'.*(1+(l_predict - l_base)./l_base);
D_b = exp(b_0)'.*(1+(b_predict - b_base)./b_base);

disp('General equilibrium local state after shock')
disp(median(D_l))

disp('General equilibrium national state after shock')
disp(median(D_b))


%General Equilibrium
lg = log(l_predict)';

bg = log(b_predict)';


%State Capacity Composite
if sigma==1
        sg = (lg.^alpha).*((tau*bg).^(1-alpha));
    else
        sg = ( (alpha.*(lg.^((sigma-1)/sigma))) + ((1-alpha).*((tau*bg).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end


%General Equilibrium
lb = log(l_base)';

bb = log(b_base)';


%State Capacity Composite
if sigma==1
        sb = (lb.^alpha).*((tau*bb).^(1-alpha));
    else
        sb = ( (alpha.*(lb.^((sigma-1)/sigma))) + ((1-alpha).*((tau*bb).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end


D_p_gen=zeros(n,num_out);


for i=1:num_out
    D_p_gen(:,i) = ( (((theta./alpha).*lg.*sg.*((lg./sg).^(1/sigma))) + (psi_2(i).*(NN*sg))) - (((theta./alpha).*lb.*sb.*((lb./sb).^(1/sigma))) + (psi_2(i).*(NN*sb))) )./(((theta./alpha).*lb.*sb.*((lb./sb).^(1/sigma))) + (psi_2(i).*(NN*sb)));
    
end

D_p_gen_l = (1+D_p_gen).*p;
D_p_gen_l_1 = D_p_gen_l(:,1);
D_p_gen_l_2 = D_p_gen_l(:,2);
D_p_gen_l_3 = D_p_gen_l(:,3);
D_p_gen_l_4 = D_p_gen_l(:,4);

D_p_gen_l_1 = D_p_gen_l_1(~isnan(D_p_gen_l_1));
D_p_gen_l_2 = D_p_gen_l_2(~isnan(D_p_gen_l_2));
D_p_gen_l_3 = D_p_gen_l_3(~isnan(D_p_gen_l_3));
D_p_gen_l_4 = D_p_gen_l_4(~isnan(D_p_gen_l_4));

disp('General Equilibrium Outcomes after shock')
disp(median([D_p_gen_l_1 D_p_gen_l_2 D_p_gen_l_3 D_p_gen_l_4]))

%%
%Normative Exercise

%baseline
l_0 = l.*ones(n,1);
b_0 = b.*ones(n,1);
s_0 = [l_0 b_0];
h_prime_b=0;

X_e = [V(:,3:5) zscore(X) (zscore(X).^2) log(pop) (log(pop).^2) X_d ones(n,1)];

%State Capacity Composite
if sigma==1
        ss = (l_0.^alpha).*((tau*b_0).^(1-alpha));
    else
        ss = ( (alpha.*(l_0.^((sigma-1)/sigma))) + ((1-alpha).*((tau*b_0).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end

f = @(x) BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g + theta*((X_e*bbetas)), zet_br, NN, zetas, h_prime_b, x) ; % function of dummy variable x

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g + theta*((X_e*bbetas)), zet_br, NN, zetas, h_prime_b, log(s_0))

options = optimset('Display','iter', 'MaxFunEvals', 500000, 'MaxIter', 1000);
[root, fval] = fsolve(f, log(s_0), options);

lg = (exp(root(1:n)))';

bg = (exp(root(n+1:2*n)))';

%State Capacity Composite
if sigma==1
        s_exper = (lg.^alpha).*((tau*bg).^(1-alpha));
    else
        s_exper = ( (alpha.*(lg.^((sigma-1)/sigma))) + ((1-alpha).*((tau*bg).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g + theta*((X_e*bbetas)), zet_br, NN, zetas, h_prime_b, root)

p_exper=zeros(n,num_out);
for i=1:num_out

    p_exper(:,i) = ((theta./alpha).*lg.*s_exper.*((lg./s_exper).^(1/sigma))) + psi_2(i)*NN*s_exper +  ([zscore(X) pop]*beta_out(((i-1)*k)+i:i*(k+1))) + (X_d*zet_out((1+((i*ndep)-ndep):i*ndep)));
    
    p_exper(:,i) = (p_exper(:,i)*std(p(:,i))) + mean(p(:,i)); 
    
end

welfare_gain_exper = (p_exper-p_base);

disp ('Baseline')

disp('Raw Average Welfare Gains from Optimal Allocation')
disp(mean(welfare_gain_exper))

disp('Median Welfare Gains from Optimal Allocation')
disp(median(welfare_gain_exper))


%Increase coeff on Betweenness
l_0 = l.*ones(n,1);
b_0 = b.*ones(n,1);
s_0 = [l_0 b_0];
h_prime_b=0;

%State Capacity Composite
if sigma==1
        ss = (l_0.^alpha).*((tau*b_0).^(1-alpha));
    else
        ss = ( (alpha.*(l_0.^((sigma-1)/sigma))) + ((1-alpha).*((tau*b_0).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end

f = @(x) BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g + theta*((X_e*(bbetas.*[1.25; ones(size(X_e,2)-1,1)]))), zet_br, NN, zetas, h_prime_b, x) ; % function of dummy variable x

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g + theta*((X_e*(bbetas.*[1.25; ones(size(X_e,2)-1,1)]))), zet_br, NN, zetas, h_prime_b, log(s_0))

options = optimset('Display','iter', 'MaxFunEvals', 500000, 'MaxIter', 1000);
[root, fval] = fsolve(f, log(s_0), options);

lg = (exp(root(1:n)))';

bg = (exp(root(n+1:2*n)))';

%State Capacity Composite
if sigma==1
        s_exper = (lg.^alpha).*((tau*bg).^(1-alpha));
    else
        s_exper = ( (alpha.*(lg.^((sigma-1)/sigma))) + ((1-alpha).*((tau*bg).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g + theta*((X_e*(bbetas.*[1.25; ones(size(X_e,2)-1,1)]))), zet_br, NN, zetas, h_prime_b, root)

p_bet=zeros(n,num_out);
for i=1:num_out

    p_bet(:,i) = ((theta./alpha).*lg.*s_exper.*((lg./s_exper).^(1/sigma))) + psi_2(i)*NN*s_exper +  ([zscore(X) pop]*beta_out(((i-1)*k)+i:i*(k+1))) + (X_d*zet_out((1+((i*ndep)-ndep):i*ndep)));
    
    p_bet(:,i) = (p_bet(:,i)*std(p(:,i))) + mean(p(:,i)); 
    
end

welfare_gain_exper = (p_bet-p_exper);

disp('Betweenness')
disp('Median Welfare Gains')
disp(median(welfare_gain_exper))


%Increase coeff on Bonacich
l_0 = l.*ones(n,1);
b_0 = b.*ones(n,1);
s_0 = [l_0 b_0];
h_prime_b=0;

%State Capacity Composite
if sigma==1
        ss = (l_0.^alpha).*((tau*b_0).^(1-alpha));
    else
        ss = ( (alpha.*(l_0.^((sigma-1)/sigma))) + ((1-alpha).*((tau*b_0).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end

f = @(x) BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g + theta*((X_e*(bbetas.*[1; 1.25; ones(size(X_e,2)-2,1)]))), zet_br, NN, zetas, h_prime_b, x) ; % function of dummy variable x

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g + theta*((X_e*(bbetas.*[1; 1.25; ones(size(X_e,2)-2,1)]))), zet_br, NN, zetas, h_prime_b, log(s_0))

options = optimset('Display','iter', 'MaxFunEvals', 500000, 'MaxIter', 1000);
[root, fval] = fsolve(f, log(s_0), options);

lg = (exp(root(1:n)))';

bg = (exp(root(n+1:2*n)))';

%State Capacity Composite
if sigma==1
        s_exper = (lg.^alpha).*((tau*bg).^(1-alpha));
    else
        s_exper = ( (alpha.*(lg.^((sigma-1)/sigma))) + ((1-alpha).*((tau*bg).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g + theta*((X_e*(bbetas.*[1; 1.25; ones(size(X_e,2)-2,1)]))), zet_br, NN, zetas, h_prime_b, root)

p_bon=zeros(n,num_out);
for i=1:num_out

    p_bon(:,i) = ((theta./alpha).*lg.*s_exper.*((lg./s_exper).^(1/sigma))) + psi_2(i)*NN*s_exper +  ([zscore(X) pop]*beta_out(((i-1)*k)+i:i*(k+1))) + (X_d*zet_out((1+((i*ndep)-ndep):i*ndep)));
    
    p_bon(:,i) = (p_bon(:,i)*std(p(:,i))) + mean(p(:,i)); 
    
end

welfare_gain_exper = (p_bon-p_exper);

disp('Bonacich')
disp('Median Welfare Gains')
disp(median(welfare_gain_exper))


%Increase coeff on Local Clustering
l_0 = l.*ones(n,1);
b_0 = b.*ones(n,1);
s_0 = [l_0 b_0];
h_prime_b=0;

%State Capacity Composite
if sigma==1
        ss = (l_0.^alpha).*((tau*b_0).^(1-alpha));
    else
        ss = ( (alpha.*(l_0.^((sigma-1)/sigma))) + ((1-alpha).*((tau*b_0).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end

f = @(x) BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g + theta*((X_e*(bbetas.*[1; 1; 1.25; ones(size(X_e,2)-3,1)]))), zet_br, NN, zetas, h_prime_b, x) ; % function of dummy variable x

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g + theta*((X_e*(bbetas.*[1; 1; 1.25; ones(size(X_e,2)-3,1)]))), zet_br, NN, zetas, h_prime_b, log(s_0))

options = optimset('Display','iter', 'MaxFunEvals', 500000, 'MaxIter', 1000);
[root, fval] = fsolve(f, log(s_0), options);

lg = (exp(root(1:n)))';

bg = (exp(root(n+1:2*n)))';

%State Capacity Composite
if sigma==1
        s_exper = (lg.^alpha).*((tau*bg).^(1-alpha));
    else
        s_exper = ( (alpha.*(lg.^((sigma-1)/sigma))) + ((1-alpha).*((tau*bg).^((sigma-1)/sigma))) ).^(sigma/(sigma-1));
end

BR_full(alpha, sigma, theta, psi_1, psi_2, tau, eta, exp_cond_g + theta*((X_e*(bbetas.*[1; 1; 1.25; ones(size(X_e,2)-3,1)]))), zet_br, NN, zetas, h_prime_b, root)

p_clus=zeros(n,num_out);
for i=1:num_out

    p_clus(:,i) = ((theta./alpha).*lg.*s_exper.*((lg./s_exper).^(1/sigma))) + psi_2(i)*NN*s_exper +  ([zscore(X) pop]*beta_out(((i-1)*k)+i:i*(k+1))) + (X_d*zet_out((1+((i*ndep)-ndep):i*ndep)));
    
    p_clus(:,i) = (p_clus(:,i)*std(p(:,i))) + mean(p(:,i)); 
    
end

welfare_gain_exper = (p_clus-p_exper);

disp('Local Clustering')
disp('Median Welfare Gains from Optimal Allocation')
disp(median(welfare_gain_exper))







