%Estimation of State Capacity Spillovers: SYSTEM GMM
%Acemoglu, Garcia-Jimeno, Robinson

clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Import data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Open data file State_Capacity_Network_Work.mat
load State_Capacity_Network_Work.mat

%%
%OPTIONS

%1: Choose Outcome variable:
 %out = 3;          %1 = Secondary enrollment 92-02
                    %2 = Services Coverage 1998
                    %3 = Not in poverty 2005
                    %4 = Life Quality Index
                    %5 = Primary enrollment 92-02 (placebo)
                    %6 = Vaccination coverage 1998 (placebo)    
                                       
%2: Choose Measure of State Capacity:
 a_or_b = 1;        %0 = State municipality agencies 
                    %1 = State municipality bureaucracies
 
%3: Choose if control for national-level bureaucracy 
 nat_bur=0;         %0 = no 
                    %1 = yes

%4: Choose if control for rivers density and land shares
 river_land=0;      %0=no
                    %1=yes

%5: Choose if control for distance to current road
current_road=1;     %0=no
                    %1=yes
                    
%6: Choose assumption on Network Links
 ass_network_structure=0;       %0 = Network link is neighbors           
                                %1 = Network link is neighbors and neighbors of neighbors 

%7: Choose order of instruments: 
 order=2;           %2 = Instruments are Nc and N2c
                    %3 = Instruments are N2c and N3c 

%8: Choose the subset of Instruments to use:
 instruments=0;     %0 = Royal roads and Colonial Agencies
                    %1 = Royal roads only
                    %2 = Colonial Agencies only

%9: Choose if exclude violent municipalities
excl_viol=0;        %0 = Do not exclude
                    %1 = Exclude from sample
                    %2 = Exclude from network

%10: Choose how to treat population
pop_treat=0;        %0=Instrument for Population with Historical Population
                    %1=Control for Population                    
%%

 %Choose the endogenous and exogenous variables for the model
 Yg_input = [Yg(:,2) ((Yg(:,4)+Yg(:,5)+Yg(:,6))/3) Yg(:,9) Yg(:,10)];    
         
     %Choose if state capacity is agencies or bureaucracies    
     if a_or_b ==0
         S_input = S(:,6)+S(:,7)+S(:,8)+S(:,11)+S(:,12)+S(:,13)+S(:,14)+S(:,15)+S(:,16)+S(:,18)+S(:,19);                 %S = Contemporary State
     else
         S_input = S(:,1);
     end
         
     C_input = [C(:,1) C(:,3)];                                               %C = Colonial State
     R_input = R;                                                             %R = Distance to royal roads
     
     %Choose controls
     if nat_bur==0 && river_land==0 && current_road==0                        %X = Controls
         X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5)];
        else if nat_bur==0 && river_land==0 && current_road==1
                X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1)];               
            else if nat_bur==0 && river_land==1 && current_road==0
                    X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) G(:,6:8) L(:,1:7) L(:,12:15)];
                else if nat_bur==0 && river_land==1 && current_road==1
                        X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1) G(:,6:8) L(:,1:7) L(:,12:15)];
                    else if nat_bur==1 && river_land==0 && current_road==0
                            X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) log(1+S(:,2))];
                        else if nat_bur==1 && river_land==0 && current_road==1
                                X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1) log(1+S(:,2))];
                            else if nat_bur==1 && river_land==1 && current_road==0
                                    X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) G(:,6:8) L(:,1:7) L(:,12:15) log(1+S(:,2))];
                                else 
                                    X_input = [Xs(:,2) DD(:,1:25) Dc G(:,1:5) Xs(:,1) G(:,6:8) L(:,1:7) L(:,12:15) log(1+S(:,2))];
                                end
                            end
                        end
                    end
                end
            end
     end        

     X_input = [X_input H(:,8)./1000 H(:,9)];                                 %To add 1843 population and missing pop dummy
         
     %Ev_input = Ev;                                                          %Historical Electoral variability
     
     
     A_input = Adj;                                                           %A = Adjacency matrix
     Dd_input = Ddist;                                                        %Original Distances matrix
     Drr_input = Drankrange;                                                  %Original Range of elevation ranks along path matrix
     Drv_input = Drankvar;                                                    %Original Variance of elevation ranks along path matrix
     Yv_input = Yv(:,16);                                                     %Original Per capita attacks   
     
     
     
     
     %Clean data from missing values
     [Yg_clean S_clean C_clean X_clean R_clean Yv_clean A_clean Dd_clean Drr_clean Drv_clean DaneCodes_clean] = clean_missings(Yg_input, S_input, C_input, X_input, R_input, Yv_input, A_input, Dd_input, Drr_input, Drv_input, DaneCodes);

 clear Adj C DD Dc Ddist Drankrange Drankvar G H L R S Xs Yg Yv Ygini Ev DaneCodes
 clear Yg_input S_input C_input X_input R_input Yv_input Ev_input A_input Dd_input Drr_input Drv_input


 global p s c r
 p = Yg_clean;                          %p=vectors of prosperity
 
 s= log(1+S_clean);                     %s=vector of state capacity  
 c = (C_clean);                         %c=vector of colonial state presence
 r = zscore(R_clean);                   %r=vector of distance to royal roads
 
 global D E
 D= Dd_clean;                           %D=distance matrix
 E= Drv_clean;                          %E=variance elevation matrix

 global X X_d pop hpop

 %Choose set of controls
   
    X = X_clean(:,28:end-2);                %matrix of controls
    
    hpop = X_clean(:,end-1:end);            %Matrix of 1843 population and missing 1843 population dummy
    X_d = X_clean(:,2:27);                  %matrix of department dummies and department capital dummy
    pop = X_clean(:,1)./1000;               %vector of population
 
 global A
 A=A_clean;                             %A=adjacency matrix (nxn) with 0s on the diagonal

 global n k ndep
 n=size(p,1);                           %number of observations
 k=size(X,2);                           %number of covariates
 ndep=size(X_d,2);                      %number of department dummies
 
 DaneCodes=DaneCodes_clean;             %Municipality codes to keep track
 
 %Create Indicator of violent municipalities (above 90th percentile of attacks)
 v=zeros(n,1);
 pc90 = prctile(Yv_clean,90);
 for j=1:n
     if Yv_clean(j,1)>=pc90
         v(j)=1;
     end
 end
 
 clear Yg_clean S_clean C_clean X_clean R_clean Yv_clean A_clean Dd_clean Drr_clean Drv_clean DaneCodes_clean
 
 
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Assumptions on the Network Structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if excl_viol==2
    p = p(~v,:);
    s = s(~v,:);
    c = c(~v,:);
    r = r(~v,:);
    X = X(~v,:);
    X_d =X_d(~v,:);
    pop = pop(~v,:);
    hpop = hpop(~v,:);
    
    D0 = D(~v,:)';
    D = (D0(~v,:))';
    
    E0 = E(~v,:)';
    E = (E0(~v,:))';
    
    A0 = A(~v,:)';
    A = (A0(~v,:))';

    clear D0 E0 A0

    n=size(p,1);
end

global NE N N_2 A_2 A_3
F=1./(1+(D.*(1+E)));

%Re-sets the diagonal to zeroes
for i=1:size(F,1)
    F(i,i)=0;
end

Q=(ones(n,n)-A).*(A*A-diag(diag(A*A)));                 %Matrix of Number of Paths to Neighbors of Neighbors who are not own Neighbors
A_2=ismember(Q,1:1:max(max(Q)));                        %Adjacency Matrix of neighbors of neighbors
A_2=+A_2;                                               % from logical to numeric

Q2=(ones(n,n)-A-A_2).*((A*A*A)-diag(diag(A*A*A)));      %Matrix of Number of Paths to Neighbors of Neighbors of Neighbors who are not own Neighbors or own Neighbors of Neighbors
A_3 = ismember(Q2,1:1:max(max(Q2)));                    %Adjacency Matrix of neighbors of neighbors of neighbors
A_3=+A_3;                                               % from logical to numeric
clear Q Q2


if ass_network_structure==1 
    A=(A+A_2);                                          %Adjacency matrix when Network link is defined by assumption b
    A_2=A_3;                                            %Adjacency matrix of neighbors of neighbors when Network link is defined by assumption b
end

%Network matrix of adjacent neighbors
NE = F.*A; 

if order==2
    N=F.*A;                                                 %Network matrix of neighbors
    N_2=F.*A_2;                                             %Network matrix of neighbors of neighbors
else if order==3
        N=F.*A_2;                                           %Network matrix of neighbors of neighbors
        N_2=F.*A_3;                                         %Network matrix of neighbors of neighbors of neighbors
    end
end
 
 %%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %SYSTEM GMM
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% %Municipality-Level FOC
% %s = f( Ns , c1 , c2, r, , X , X_d)
%
% %Outcomes Equation
% %y = f(s^2, N*s, X , X_d)
% 
 %%%%%%First stage GMM
 
 assum_f_delta=0;    % if 0 we are using Assumption f(D,E), 1 if we are using assumption f(D,E,delta).
 options=optimset('Display','iter','MaxIter',40000,'MaxFunEvals',40000,'Algorithm','interior-point');
 biweight_kernel=0;                                      %0 for normal kernel, 1 for biweight kernel
 truncation_normal=160;                                  %Truncation for normal kernel: the lower the value, the stronger the truncation
 nobin=1;                                                %0 for using bins , 1 for using directly the grid of observations
 %%%%%%%%%%%%%%%
 %NOTE: We have noticed that a 40000 iteration limit for convergence for the first step estimation binds under some machines and versions of Matlab.
 %      In such case the user can increase the iteration limit in line 245 and ensure convergence of the first step estimator. This may lead to small 
 %      discrepancies in the final (second step) estimates of Columns 4/8 of Table 3 and columns 4/8/12/16 of Tables 4A and 4B (the system GMM estimamtes).
 %      Discrepancies arise in the second decimal places of the estimates only.
 %%%%%%%%%%%%%%%
 
 num_out= size(p,2);
 num_eq = num_out+1;  %One equation for each outcome plus the Best Response equation
 
 global d_bar e_bar
     number_of_neighbors=sum(A,2);
     d_bar=(1/1)*(number_of_neighbors).^(-1).*sum(A.*D,2);
     e_bar=(1/1)*(number_of_neighbors).^(-1).*sum(A.*E,2);

     %Initialize parameter value for delta
      ddelta_guess = 1*ones(1,2);    
      
 if assum_f_delta==1     % if N(d) depends on delta (the neighbors matrices are assumed to depend on a parameter vector delta)
          
      ddelta=ddelta_guess;
      [ N_d, N_d_2, F_d] = neighbors_matrix( ddelta, order );  % call a function that generates the neighbors and neighbors of neighbors matrices which depend on delta
      NN=N_d;
      NN_2=N_d_2;    
     
      Z_out_w=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (NN*r).^3 (NN*log(1+c)).^4 (NN*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop zscore(d_bar) zscore(e_bar)];
      Z_br_w=[NN*log(1+c) NN*r NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2) zscore(d_bar) zscore(e_bar)];
                 
      lb = [  0   0 -Inf*ones(1,num_out + num_out*(k+1+ndep) + 3 + k + 1 + ndep)    0    0]';
      ub = [Inf Inf  Inf*ones(1,num_out + num_out*(k+1+ndep) + 3 + k + 1 + ndep)  Inf  Inf]';    
 else
     NN=N;
     NN_2=N_2;
          
     Z_out_w=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (NN*r).^3 (NN*log(1+c)).^4 (NN*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop];
     Z_br_w=[NN*log(1+c) NN*r NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2)];
                 
       
     lb = [  0   0 -Inf*ones(1,num_out + num_out*(k+1+ndep) + 3 + k + 1 + ndep)]';
     ub = [Inf Inf  Inf*ones(1,num_out + num_out*(k+1+ndep) + 3 + k + 1 + ndep)]';  
 end
 
 num_mom = (num_out*size(Z_out_w,2))+size(Z_br_w,2); %total number of moments
 
 %Instruments for the Weighting  Matrix
    Zw = zeros(num_eq,num_mom,n);
    for obs_n=1:n
        Zw(:,:,obs_n) = [kron(eye(num_out),Z_out_w(obs_n,:)) zeros(num_out,size(Z_br_w,2)); zeros(1,(num_out*size(Z_out_w,2))) Z_br_w(obs_n,:)];
    end  
  
    ZOZ=zeros(num_mom,num_mom);
 
    for i=1:n
        ZOZ = Zw(:,:,i)'*Zw(:,:,i) + ZOZ;
    end

    %Initialize parameter values
      
      theta_guess = 1;      
      psi_1_guess = 1;
      psi_2_guess = 0.01*ones(1,num_out);
      beta_out_guess = 0*ones(1,num_out*(k+1));
      zet_out_guess = 0*ones(1,num_out*ndep);
      phi_guess = 0.01*ones(1,3);
      beta_br_guess = 0.01*ones(1,k+1);
      zet_br_guess = 0.01*ones(1,ndep);

      if assum_f_delta==1
          guess_gmm = [theta_guess psi_1_guess psi_2_guess beta_out_guess zet_out_guess phi_guess beta_br_guess zet_br_guess ddelta_guess];   
          tic
            [param_gmm_first, fval_first] = fmincon(@(param_gmm) sys_gmm_objective_function(param_gmm, assum_f_delta, truncation_normal, nobin, biweight_kernel, order, num_out, ZOZ ), guess_gmm, [], [], [], [], lb, ub, [], options);
          toc
      
      else
          guess_gmm = [theta_guess psi_1_guess psi_2_guess beta_out_guess zet_out_guess phi_guess beta_br_guess zet_br_guess];
          tic
            [param_gmm_first, fval_first] = fmincon(@(param_gmm) sys_gmm_objective_function(param_gmm, assum_f_delta, truncation_normal, nobin, biweight_kernel, order, num_out, ZOZ ), guess_gmm, [], [], [], [], lb, ub, [], options);
          toc
      end
  
 
 %%%%%%Second stage (efficient) System GMM
      
      theta = param_gmm_first(1);
      psi_1 = param_gmm_first(2);
      psi_2 = param_gmm_first(3:3+num_out-1);
      beta_out = param_gmm_first(3+num_out:3+num_out+(num_out*(k+1))-1)';
      zet_out = param_gmm_first(3+num_out+(num_out*(k+1)):3+num_out+(num_out*(k+1))+(num_out*ndep)-1)';
      phi = param_gmm_first(3+num_out+(num_out*(k+1))+(num_out*ndep):3+num_out+(num_out*(k+1))+(num_out*ndep)+3-1)';
      beta_br = param_gmm_first(3+num_out+(num_out*(k+1))+(num_out*ndep)+3:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1-1)';
      zet_br = param_gmm_first(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1)';     
      
if assum_f_delta==1                                            % if N(d) depends on delta (the neighbors matrices are assumed to depend on a parameter vector delta)      
      ddelta = param_gmm_first(end-1:end);   
      [ N_d, N_d_2, F_d] = neighbors_matrix( ddelta, order );  % call a function that generates the neighbors and neighbors of neighbors matrices which depend on delta
      NN=N_d;
      NN_2=N_d_2;
      
      Z_out_w=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop zscore(d_bar) zscore(e_bar)];
      Z_br_w=[NN*log(1+c) NN*r NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2) zscore(d_bar) zscore(e_bar)];
      
else
     NN=N;
     NN_2=N_2;
     
     Z_out_w=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop];
     Z_br_w=[NN*log(1+c) NN*r NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2)];
end      

%Non parametric conditional expectation
    pphi=[phi; beta_br];
    xx=[log(1+c) r zscore(X) log(pop)];    
    exp_cond_g= nonparametric_conditional( s - ((psi_1/theta)*(NN*s)) - (X_d*zet_br), xx, biweight_kernel, truncation_normal, pphi, nobin );  % It gives the conditional expectation of s+ppsi_gmm*N*s-X*bbeta_gmm conditional on c and lambda for each bin

    %Residuals:
    Qt = zeros(n,num_out);
    for i=1:num_out
        Qt(:,i) = zscore(p(:,i)) - (theta*(s.^2)) - (psi_2(i)*(NN*s)) - ([zscore(X) pop]*beta_out(((i-1)*k)+i:i*(k+1))) - (X_d*zet_out(1+((i*ndep)-ndep):i*ndep)); 
    end
        q_br = s - ((psi_1/theta)*(NN*s)) - (X_d*zet_br) - exp_cond_g;
        Qt = [Qt  q_br];
        Q=Qt';
        
   
    %Construction of spatially correlated weighting matrix    
    OM0=zeros(num_eq,num_eq,n);
    for obs_n=1:n
        OM0(:,:,obs_n) =Q(:,obs_n)*Q(:,obs_n)';
    end    
    
    OM1=zeros(num_eq,num_eq,n);
    for i=1:n
        neigh_i=0;
        VUi=zeros(num_eq,num_eq,n);
        for j=1:n
            if A(i,j)==1
                neigh_i=neigh_i+1;
                VUi(:,:,j)=Q(:,i)*Q(:,j)';
            end
        end
        OM1(:,:,i)=(1/neigh_i)*sum(VUi,3);
    end
    
    OM2=zeros(num_eq,num_eq,n);
    for i=1:n
        neigh2_i=0;
        VU2i=zeros(num_eq,num_eq,n);
        for j=1:n
            if A_2(i,j)==1
                neigh2_i=neigh2_i+1;
                VU2i(:,:,j)=Q(:,i)*Q(:,j)';
            end
        end
        OM2(:,:,i)=(1/neigh2_i)*sum(VU2i,3);
    end
  
    %Instruments for the Weighting  Matrix
        Zw = zeros(num_eq,num_mom,n);
        for obs_n=1:n
            Zw(:,:,obs_n) = [kron(eye(num_out),Z_out_w(obs_n,:)) zeros(num_out,size(Z_br_w,2)); zeros(1,(num_out*size(Z_out_w,2))) Z_br_w(obs_n,:)];
        end  
        
    ZOZ=zeros(num_mom,num_mom);
    for i=1:n
        ZOZ = Zw(:,:,i)'*(OM0(:,:,i) +((2/23)*(OM1(:,:,i) + OM1(:,:,i)')) + ((1/23)*(OM2(:,:,i) + OM2(:,:,i)')) )*Zw(:,:,i) + ZOZ;
    end
    
 guess_gmm = param_gmm_first;
 options=optimset('Display','iter','MaxIter',200000,'MaxFunEvals',200000,'Algorithm','interior-point');
         tic
         [param_gmm_final, fval_final] = fmincon(@(param_gmm) sys_gmm_objective_function(param_gmm, assum_f_delta, truncation_normal, nobin, biweight_kernel, order, num_out, ZOZ ), guess_gmm, [], [], [], [], lb, ub, [], options);
         toc
 
         
      theta = param_gmm_final(1);
      psi_1 = param_gmm_final(2);
      psi_2 = param_gmm_final(3:3+num_out-1);
      beta_out = param_gmm_final(3+num_out:3+num_out+(num_out*(k+1))-1)';
      zet_out = param_gmm_final(3+num_out+(num_out*(k+1)):3+num_out+(num_out*(k+1))+(num_out*ndep)-1)';
      phi = param_gmm_final(3+num_out+(num_out*(k+1))+(num_out*ndep):3+num_out+(num_out*(k+1))+(num_out*ndep)+3-1)';
      beta_br = param_gmm_final(3+num_out+(num_out*(k+1))+(num_out*ndep)+3:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1-1)';
      zet_br = param_gmm_final(3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1:3+num_out+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1)';
      
      
 if assum_f_delta==1                                            % if N(d) depends on delta (the neighbors matrices are assumed to depend on a parameter vector delta)      
      ddelta = param_gmm_final(end-1:end); 
      [ N_d, N_d_2, F_d] = neighbors_matrix( ddelta, order );  % call a function that generates the neighbors and neighbors of neighbors matrices which depend on delta
      NN=N_d;
      NN_2=N_d_2; 
 else
     NN=N;
     NN_2=N_2;
 end      

disp('parameters estimates for System GMM')
disp('theta')
disp(theta)
disp('psi1')
disp(psi_1)
disp('psi_2')
disp(psi_2)
disp('phi')
disp(phi')
if assum_f_delta==1
    disp('delta')
    disp(ddelta')
end

%%
%%%% STANDARD ERRORS %%%%
 
    if assum_f_delta==1     %if 0 we are using Assumption f(D,E), 1 if we are using assumption f(D,E,delta).   
     DF(:,:,1)=-D.*(1+ddelta(2)*E).*(F_d).^(-2);
     DN(:,:,1)=A.*DF(:,:,1);
     DN_2(:,:,1)=A_2.*DF(:,:,1);
     DF(:,:,2)=-ddelta(1)*D.*E.*(F_d).^(-2);
     DN(:,:,2)=A.*DF(:,:,2);
     DN_2(:,:,2)=A_2.*DF(:,:,2);
     DZ_out_del(:,:,1)=[DN(:,:,1)*log(1+c) DN(:,:,1)*r (DN(:,:,1)*log(1+c)).^2 (DN(:,:,1)*r).^2 (DN(:,:,1)*log(1+c)).^3 (DN(:,:,1)*r).^3 (DN(:,:,1)*log(1+c)).^4 (DN(:,:,1)*r).^4 DN_2(:,:,1)*log(1+c) DN_2(:,:,1)*r (DN_2(:,:,1)*log(1+c)).^2 (DN_2(:,:,1)*r).^2 (DN_2(:,:,1)*log(1+c)).^3 (DN_2(:,:,1)*r).^3 (DN_2(:,:,1)*log(1+c)).^4 (DN_2(:,:,1)*r).^4];
     DZ_out_del(:,:,2)=[DN(:,:,2)*log(1+c) DN(:,:,2)*r (DN(:,:,2)*log(1+c)).^2 (DN(:,:,2)*r).^2 (DN(:,:,2)*log(1+c)).^3 (DN(:,:,2)*r).^3 (DN(:,:,2)*log(1+c)).^4 (DN(:,:,2)*r).^4 DN_2(:,:,2)*log(1+c) DN_2(:,:,2)*r (DN_2(:,:,2)*log(1+c)).^2 (DN_2(:,:,2)*r).^2 (DN_2(:,:,2)*log(1+c)).^3 (DN_2(:,:,2)*r).^3 (DN_2(:,:,2)*log(1+c)).^4 (DN_2(:,:,2)*r).^4];
     DZ_br_del(:,:,1)=[DN(:,:,1)*log(1+c) DN(:,:,1)*r DN_2(:,:,1)*log(1+c) DN_2(:,:,1)*r];
     DZ_br_del(:,:,2)=[DN(:,:,2)*log(1+c) DN(:,:,2)*r DN_2(:,:,2)*log(1+c) DN_2(:,:,2)*r];
     
      Z_out=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop zscore(d_bar) zscore(e_bar)];
      Z_br=[NN*log(1+c) NN*r NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2) zscore(d_bar) zscore(e_bar)];
    else
        Z_out=[NN*log(1+c) NN*r (NN*log(1+c)).^2 (NN*r).^2 (NN*log(1+c)).^3 (N*r).^3 (N*log(1+c)).^4 (N*r).^4 NN_2*log(1+c) NN_2*r (NN_2*log(1+c)).^2 (NN_2*r).^2 (NN_2*log(1+c)).^3 (NN_2*r).^3 (NN_2*log(1+c)).^4 (NN_2*r).^4 zscore(X) X_d hpop];
        Z_br=[NN*log(1+c) NN*r NN_2*log(1+c) NN_2*r log(1+c) r zscore(X) X_d log(hpop(:,1)) hpop(:,2)];
    end
 
    num_par=size(param_gmm_final,2);
    mom_br =size(Z_br,2);
    mom_out=size(Z_out,2);

%Construction of Z_Dq_Arr

    Zi_Arr = zeros(num_eq,num_mom,n);
        for obs_n=1:n
            Zi_Arr(:,:,obs_n) = [kron(eye(num_out),Z_out(obs_n,:)) zeros(num_out,size(Z_br,2)); zeros(1,(num_out*size(Z_out,2))) Z_br(obs_n,:)];
        end    
    
   %Non parametric conditional expectation
    pphi=[phi; beta_br];
    xx=[log(1+c) r zscore(X) log(pop)];    
    exp_cond_g= nonparametric_conditional( s - ((psi_1/theta)*(NN*s)) - (X_d*zet_br), xx, biweight_kernel, truncation_normal, pphi, nobin );  % It gives the conditional expectation of s+ppsi_gmm*N*s-X*bbeta_gmm conditional on c and lambda for each bin

    %Residuals:
    Qt = zeros(n,num_out);
    for i=1:num_out
        Qt(:,i) = zscore(p(:,i)) - (theta*(s.^2)) - (psi_2(i)*(NN*s)) - ([zscore(X) pop]*beta_out(((i-1)*k)+i:i*(k+1))) - (X_d*zet_out(1+((i*ndep)-ndep):i*ndep)); 
    end
        q_br = s - ((psi_1/theta)*(NN*s)) - (X_d*zet_br) - exp_cond_g;
        Qt = [Qt  q_br];
        Q=Qt';
 
        
    %Construction of spatially correlated weighting matrix    
    OM0=zeros(num_eq,num_eq,n);
    for obs_n=1:n
        OM0(:,:,obs_n) =Q(:,obs_n)*Q(:,obs_n)';
    end    
    
    OM1=zeros(num_eq,num_eq,n);
    for i=1:n
        neigh_i=0;
        VUi=zeros(num_eq,num_eq,n);
        for j=1:n
            if A(i,j)==1
                neigh_i=neigh_i+1;
                VUi(:,:,j)=Q(:,i)*Q(:,j)';
            end
        end
        OM1(:,:,i)=(1/neigh_i)*sum(VUi,3);
    end
    
    OM2=zeros(num_eq,num_eq,n);
    for i=1:n
        neigh2_i=0;
        VU2i=zeros(num_eq,num_eq,n);
        for j=1:n
            if A_2(i,j)==1
                neigh2_i=neigh2_i+1;
                VU2i(:,:,j)=Q(:,i)*Q(:,j)';
            end
        end
        OM2(:,:,i)=(1/neigh2_i)*sum(VU2i,3);
    end
  
    %Instruments for the Weighting  Matrix
        Zw = zeros(num_eq,num_mom,n);
        for obs_n=1:n
            Zw(:,:,obs_n) = [kron(eye(num_out),Z_out_w(obs_n,:)) zeros(num_out,size(Z_br_w,2)); zeros(1,(num_out*size(Z_out_w,2))) Z_br_w(obs_n,:)];
        end  
        
    ZWZ=zeros(num_mom,num_mom);
    for i=1:n
        ZWZ = Zw(:,:,i)'*(OM0(:,:,i) +((2/23)*(OM1(:,:,i) + OM1(:,:,i)')) + ((1/23)*(OM2(:,:,i) + OM2(:,:,i)')) )*Zw(:,:,i) + ZWZ;
    end    
        
     
 %Average derivative of g(phi*c + X*b):
 g=[xx*pphi exp_cond_g];
 
 b_g = ([ones(n,1) g(:,1)]'*[ones(n,1) g(:,1)])\([ones(n,1) g(:,1)]'*g(:,2));
 g_prime=b_g(2);
 
 %Construction of Dq   (Gradient of equations)
 Dq=zeros(num_eq,num_par,n);
 for j=1:num_out
    Dq(j,1,:)=-(s.^2)';
    Dq(j,2,:)=zeros(n,1);
    
    if j==1
        Dq(j,3,:)=-NN*s;
        Dq(j,4,:)=zeros(n,1);
        Dq(j,5,:)=zeros(n,1);   
        Dq(j,6,:)=zeros(n,1);
    else if j==2
            Dq(j,3,:)=zeros(n,1);
            Dq(j,4,:)=-NN*s;
            Dq(j,5,:)=zeros(n,1);   
            Dq(j,6,:)=zeros(n,1);
        else if j==3
                Dq(j,3,:)=zeros(n,1);
                Dq(j,4,:)=zeros(n,1);
                Dq(j,5,:)=-NN*s;   
                Dq(j,6,:)=zeros(n,1);
            else if j==4
                    Dq(j,3,:)=zeros(n,1);
                    Dq(j,4,:)=zeros(n,1);
                    Dq(j,5,:)=zeros(n,1);   
                    Dq(j,6,:)=-NN*s;
                end
            end
        end
    end
    
    Dq(j,7+((j-1)*(k+1)):7+(j*(k+1))-1,:)=-[zscore(X) pop]';
    Dq(j,7+(num_out*(k+1))+((j-1)*ndep):7+(num_out*(k+1))+(j*ndep)-1,:)=-X_d';
    Dq(j,7+(num_out*(k+1))+(num_out*ndep):7+(num_out*(k+1))+(num_out*ndep)+3-1,:)=zeros(n,3)';
    Dq(j,7+(num_out*(k+1))+(num_out*ndep)+3:7+(num_out*(k+1))+(num_out*ndep)+3+k+1-1,:)=zeros(n,k+1)';
    Dq(j,7+(num_out*(k+1))+(num_out*ndep)+3+k+1:7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1,:)=zeros(n,ndep)';
    if assum_f_delta==1
        Dq(j,end-1,:)=-psi_2(j)*(DN(:,:,1)*s);
        Dq(j,end,:)=-psi_2(j)*(DN(:,:,2)*s);
    end
 end
 
    Dq(num_out+1,1,:)=(psi_1/(theta^2))*(NN*s);
    Dq(num_out+1,2,:)=-(1/theta)*(NN*s);
    Dq(num_out+1,3,:)=zeros(n,1);
    Dq(num_out+1,4,:)=zeros(n,1);
    Dq(num_out+1,5,:)=zeros(n,1);
    Dq(num_out+1,6,:)=zeros(n,1);
    Dq(num_out+1,7:7+(num_out*(k+1))-1,:)=zeros(n,num_out*(k+1))';
    Dq(num_out+1,7+(num_out*(k+1)):7+(num_out*(k+1))+(num_out*ndep)-1,:)=zeros(n,num_out*ndep)';
    Dq(num_out+1,7+(num_out*(k+1))+(num_out*ndep):7+(num_out*(k+1))+(num_out*ndep)+3-1,:)=-xx(:,1:3)'*g_prime;
    Dq(num_out+1,7+(num_out*(k+1))+(num_out*ndep)+3:7+(num_out*(k+1))+(num_out*ndep)+3+k+1-1,:)=-[zscore(X) log(pop)]'*g_prime;
    Dq(num_out+1,7+(num_out*(k+1))+(num_out*ndep)+3+k+1:7+(num_out*(k+1))+(num_out*ndep)+3+k+1+ndep-1,:)=-X_d'*g_prime;
    if assum_f_delta==1
        Dq(num_out+1,end-1,:)=-(psi_1/theta)*((DN(:,:,1)*s));
        Dq(num_out+1,end,:)=-(psi_1/theta)*((DN(:,:,2)*s));
    end
 
 
 Z_Dq_Arr=zeros(num_mom,num_par,n);
 for i=1:n
     Z_Dq_Arr(:,:,i) = Zi_Arr(:,:,i)'*Dq(:,:,i);
 end
    
   
%Construction of DZ_q_Arr

    %Construction of DZ (Gradient of Instruments)
    DZout=zeros(mom_out,num_par,n);
    if assum_f_delta==1
        for i=1:size(DZ_out_del,2)
            DZout(i,end-1,:)= DZ_out_del(:,i,1);
        end
        for i=1:size(DZ_out_del,2)
            DZout(i,end,:)= DZ_out_del(:,i,2);
        end
    end
        
    DZbr=zeros(mom_br,num_par,n);
    if assum_f_delta==1    
        for i=1:size(DZ_br_del,2) 
            DZbr(i,end-1,:)= DZ_br_del(:,i,1);
        end 
        for i=1:size(DZ_br_del,2)
            DZbr(i,end,:)= DZ_br_del(:,i,2);
        end
    end
 
 
    DZ_q_Arr=zeros(num_mom,num_par,n);
    if assum_f_delta==1
    for i=1:n
        DZ_q_Arr(:,:,i)=[repmat(DZout(:,:,i),num_out,1); DZbr(:,:,i)].*(repmat([reshape(repmat(Q(1:num_out,i)',mom_out,1),mom_out*num_out,1) ; Q(end,i)*ones(mom_br,1)],1,num_par));
    end
    end
    
 %Construction of DG
 DG_Arr = Z_Dq_Arr + DZ_q_Arr;   
    
 DG = sum(DG_Arr,3);
 
 %Variance Covariance Matrix of the GMM estimator
 Avar_gmm = inv((DG'/ZWZ)*DG);   
 std_errors_gmm = sqrt(diag(Avar_gmm));
 disp('Standar Errors for Parameter Estimates System GMM')
 disp('standard error theta')
 disp(std_errors_gmm(1)')
 disp('standard error psi_1')
 disp(std_errors_gmm(2)')
 disp('standard errors psi_2')
 disp(std_errors_gmm(3:6)')
 disp('standard error phi')
 disp(std_errors_gmm(3+num_out+(num_out*k)+(num_out*ndep):3+num_out+(num_out*k)+(num_out*ndep)+3-1)')
 if assum_f_delta==1
  disp('standard errors delta')
  disp(std_errors_gmm(end-1:end)')
 end
 

%%
%MARGINAL EFFECTS
%Best Response Marginal Effects
    Avg_nij = NN(:);
    Avg_nij =  mean(Avg_nij(Avg_nij~=0));
    
    
    disp('ds_i/ds_j ')
    disp(Avg_nij*psi_1/theta)
    disp('standard error')
    disp(sqrt(Avg_nij^2*( (psi_1^2/theta^2)*( (Avar_gmm(2,2)/(psi_1^2)) - (2*Avar_gmm(1,2)/(psi_1*theta)) + (Avar_gmm(1,1)/(theta^2)) ) ))) %Delta method for the variance of a quotient


    disp('Marginal effects phi')
    disp(g_prime*phi')
    disp('standard errors')
    disp(abs(g_prime)*std_errors_gmm(3+num_out+(num_out*k)+(num_out*ndep):3+num_out+(num_out*k)+(num_out*ndep)+3-1)')


%Prosperity equations marginal Effects   
    disp('Own Marginal Effect dp_i/ds_i')
    disp(2*theta*(mean(s)))   
    disp('standard error')
    disp(sqrt(2*mean(s))*std_errors_gmm(1))
    
    
    disp('Neighbors Marginal Effect dp_i/ds_j')
    disp(Avg_nij*psi_2)
    disp('Standard Errors')
    disp(Avg_nij*std_errors_gmm(3:6)')


%%    
 %Condition for Unique Equilibrium:
 disp('abs(lambda_min)')
 disp(abs(min(eig(NN))))
 
 disp('(psi_1/theta)^-1')
 disp(theta/psi_1)
 
 if abs(min(eig(NN)))<theta/psi_1
     disp('Unique Equilibrium Condition Satisfied')
 else
     disp('Unique Equilibrium Condition Not Satisfied')
 end
     
 
%%
%%Model Fit
  
%Observed and Predicted State  
  S_S_hat = [DaneCodes  s  ( ((psi_1/theta)*(NN*s)) + (X_d*zet_br) + g(:,2))];
  
%Observed and Predicted Prosperity  
  P_P_hat=zeros(n,3,num_out);
  for j=1:num_out
      P_P_hat(:,:,j) = [DaneCodes zscore(p(:,j)) (theta*(s.^2)) + (psi_2(j)*(NN*s)) + ([zscore(X) pop]*beta_out(((j-1)*k)+j:j*(k+1))) + (X_d*zet_out(1+((j*ndep)-ndep):j*ndep))];
  end


%%
%Experiment
%Move all state capcity below median to the median

s_new = max(median(s),s);

s_shock = max(median(s),s)-s;

s_hat=zeros(n,num_out);

Dp=zeros(n,num_out);
p_new=zeros(n,num_out);
p_hat=zeros(n,num_out);
Dpstd=zeros(n,num_out);
direct=zeros(1,num_out);
spillovers=zeros(1,num_out);

for i=1:num_out
    %Partial Equilibrium Experiment
    
    if i==1
        disp('SECONDARY ENROLLMENT')
    else if i==2
            disp('PUBLIC UTILITIES')
        else if i==3
                disp('NOT IN POVERTY')
            else if i==4
                    disp('LIFE QUALITY INDEX')
                end
            end
        end
    end             
    disp('Partial Equilibrium')    
      
Dp(:,i) = (theta*exp_cond_g.*s_shock) + (psi_1*( (s_new.*(NN*s_new)) - (s.*(NN*s))  )) + (psi_2(i)*(NN*s_shock));
    
p_hat(:,i)= (theta*exp_cond_g.*s) + (psi_1*s.*(NN*s)) + (psi_2(i)*(NN*s));

Dpstd(:,i) = Dp(:,i)/std(p_hat(:,i));

p_new(:,i) = zscore(p(:,i)) + Dpstd(:,i);

disp('Initial Outcome median')
disp(median(p(:,i)))
disp('Initial Outcome mean')
disp(mean(p(:,i)))
disp('Experiment Outcome median')
disp(median( std(p(:,i))*p_new(:,i) + mean(p(:,i)) ))
disp('Experiment Outcome mean')
disp(mean( std(p(:,i))*p_new(:,i) + mean(p(:,i)) ))

disp('Experiment State mean')
disp(mean(exp(s_new))-1)

%Fraction of change from direct effect:

direct = sum( (theta*exp_cond_g.*s_shock) + (psi_1*(s.*(NN*s_shock))) )/sum(Dp(:,i));

%Fraction of chnage from spillovers

spillovers = sum( (psi_1*( (s_shock.*(NN*s)) +  (s_shock.*(NN*s_shock)) ) ) + (psi_2(i)*(NN*s_shock) )  )/sum(Dp(:,i));

disp('Fraction Direct')
disp(direct)
disp('Fraction Spillovers')
disp(spillovers)
    
    
%General equilibrium Experiment
s_hat = (eye(n) - (psi_1/theta)*NN)\exp_cond_g;
s_gen_eq = s_hat +  ((eye(n) - (psi_1/theta)*NN)\s_shock);

disp('General Equilibrium')
disp('Initial State Presence median')
disp(exp(median(s))-1)
disp('Initial State Presence mean')
disp(mean(exp(s)-1))
disp('Experiment State median')
disp(exp(median(s_gen_eq))-1)
disp('Experiment State mean')
disp(mean(exp(s_gen_eq))-1)

%Can use Either definition of p:
p_new(:,i) = (theta*(s_gen_eq.^2)) + (psi_2(i)*(NN*s_gen_eq));
%p_new(:,i) = (theta*exp_cond_g.*s_gen_eq) + (psi_1*s_gen_eq.*(NN*s_gen_eq)) + (psi_2(i)*(NN*s_gen_eq));

disp('Initial Outcome median')
disp(median(p(:,i)))
disp('Initial Outcome mean')
disp(mean(p(:,i)))
disp('Experiment Outcome median')
disp(median( std(p(:,i))*p_new(:,i) + mean(p(:,i)) ))
disp('Experiment Outcome mean')
disp(mean( std(p(:,i))*p_new(:,i) + mean(p(:,i)) ))
    
end














